package display;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;

import mPack.Estimator;
import mPack.Motion;
import mPack.SFM_Model;
import mPack.SenseData;
import mPack.Truth;
import utilPack.Util;

public class CaptionArea extends Canvas{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private IFrame frame;
	
	private SFM_Model model;
	
	private Truth truth;
	
	private SenseData sense;
	
	private Estimator estimator;
	
	private Motion motion;

	// values are 0,1,2 for left, centre and right areas
	private int actor;
	   
    //--------------------------------------------------------------------------------------
    //                              Constructor
    //--------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param vf
	 * @param i
	 */
	public CaptionArea(IFrame vf, SFM_Model model, Truth truth, SenseData sense, Estimator estimator, Motion motion,int i)
	{
		this.frame = vf;
		this.model = model;
		this.truth = truth;
		this.sense = sense;
		this.estimator = estimator;
		this.motion = motion;
		actor = i;
		setVisible(true);	
	}
	
	
	//---------------------------------------------------------------------------------------
	//                                         drawing
	//---------------------------------------------------------------------------------------
	
	public void paint(Graphics g) 
	{
		 			 
		 setBackground(Color.WHITE);
		 int corner = 1;
		 int pixels = 511;
		 int depth = frame.captionDepth;

		 // clear out any previous captions
		 g.setColor(Color.WHITE);
		 g.fillRect(corner,  corner,  pixels, depth -2);
		 g.setColor(Color.BLACK);
		 g.drawRect(corner,  corner,  pixels, depth - 2);
		 
		 setFont(g,0);
		 
		 drawCaptions(g);

	}
	
	/**
	 * 
	 * @param g
	 * @param type
	 */
	private void setFont(Graphics g, int type)
	{
		if (type == 0) g.setFont(new Font("Arial",Font.PLAIN,12));
		if (type == 1) g.setFont(new Font("Arial",Font.BOLD,14));
	}
	
	/**
	 * 
	 * @param g
	 */
	private void drawCaptions(Graphics g)
	{
		int caption = model.getCaption(actor);
		
		
		if (caption == SFM_Model.BEE) writeBeeCaptions(g);
		if (caption == SFM_Model.BAT) writeBatCaptions(g);

		if (caption == SFM_Model.MOTION)  writeMotionCaption(g);
		if (caption == SFM_Model.ACCURACY)  writeAccuracyCaption(g);
		if (caption == SFM_Model.RANGEGRAPH)  writeRangeGraphCaption(g);
	}
	
	// methods to display the 3-D model. 'Mean' is not used. The index of the array is model.estMethod
	String[] displayMethod = {"Full Bayes","Mean","Tracking","Noisy Tracking"};

	//------------------------------------------------------------------------------------------------------------------------------------
	//
	//------------------------------------------------------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param g
	 */
	private void writeBeeCaptions(Graphics g)
	{
		String species = "Bee";
		topLine(g,species);
		
		writeAcuity(g,1,0);
		
		thirdLine(g);
	}
	
	/**
	 * 
	 * @param g
	 */
	private void writeBatCaptions(Graphics g)
	{
		String species = "Bat";
		topLine(g,species);

		
		String echoRes = "Echo res: " + Util.round(sense.range_err, 2);
		drawText(g,1,0,echoRes);

		String dopplerRes = "Doppler res: " + Util.round(sense.getErrBee(), 2);
		drawText(g,1,1,dopplerRes);
		
		
		thirdLine(g);
	}


	//------------------------------------------------------------------------------------------------------------------------------------
	//
	//------------------------------------------------------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param g
	 * @param species
	 */
	private void topLine(Graphics g, String species)
	{
		// Big bold font
		setFont(g,1);
		drawText(g,0,0,species);
		setFont(g,0);
		
		String meth = displayMethod[estimator.estTypeDrawn];
		drawText(g,0,1,meth);
		
		writeDecay(g,0,2);
	}
	
	/**
	 * 
	 * @param g
	 */
	private void thirdLine(Graphics g)
	{
		String angle = Integer.valueOf((int)model.omega).toString();
		String rot = "Rotate: " + angle + " deg.";
		drawText(g,2,0,rot);
		int drawType = estimator.estTypeDrawn;

		if (drawType == estimator.FULLBAYES) writeStepsUsed(g,2,1);
		
		else if (drawType == estimator.NOISYTRACK ) writeNoiseLevel(g,2,1);
	}
	

	//------------------------------------------------------------------------------------------------------------------------------------
	//
	//------------------------------------------------------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param g
	 */
	private void writeMotionCaption(Graphics g)
	{
		setFont(g,1);
		String title= "Probability of detecting a moving shape (" + truth.shapeName + ")";
		drawText(g,0,0,title);
		setFont(g,0);
				
		writeAcuity(g,1,0);
		writeDecay(g,1,1);
		writeObjects(g,1,2);
		
		writeNoiseLevel(g,2,0);
		String motion = "Total motion: " + Util.round(truth.motion, 2);
		drawText(g,2,2,motion);
	}
	
	/**
	 * 
	 * @param g
	 */
	private void writeAccuracyCaption(Graphics g)
	{
		setFont(g,1);
		String title= "Average of ratio: (position error/range from bee) for all points";
		drawText(g,0,0,title);
		setFont(g,0);
				
		writeAcuity(g,1,0);
		writeDecay(g,1,1);
		writeObjects(g,1,2);
		
		writeNoiseLevel(g,2,0);
		String motion = "(motion of shape) " + Util.round(truth.motion, 2);
		if (truth.isShape) drawText(g,2,1,motion);
	}

	/**
	 * 
	 * @param g
	 */
	private void writeRangeGraphCaption(Graphics g)
	{
		setFont(g,1);
		String title= "Likelihood vs. range for point " + model.selectedObject;
		drawText(g,0,0,title);
		setFont(g,0);
				
		writeAcuity(g,1,0);
		writeDecay(g,1,1);
		
		writeNoiseLevel(g,2,0);
		String truth = "| = true range";
		drawText(g,2,1,truth);
		
	}


	//------------------------------------------------------------------------------------------------------------------------------------
	//
	//------------------------------------------------------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param g
	 * @param row
	 * @param col
	 */
	private void writeAcuity(Graphics g, int row, int col)
	{
		String visRes = "Visual acuity: " + Util.round(sense.getErrBee(), 2) + " degree";
		drawText(g,row,col,visRes);
		
	}
	
	/**
	 * 
	 * @param g
	 * @param row
	 * @param col
	 */
	private void writeObjects(Graphics g, int row, int col)
	{
		int objs = (int)(truth.randomPoints);
		String oString = Integer.valueOf(objs) + " static points";
		drawText(g,row,col,oString);
	}
	
	/**
	 * 
	 * @param g
	 * @param row
	 * @param col
	 */
	private void writeNoiseLevel(Graphics g, int row, int col)
	{
		double noiseLevel = Math.exp(-estimator.mem_noise*Math.log(10.0));
		String memNoise = estimator.noiseDimension + "-D ";
		if (estimator.relative) memNoise = memNoise + "relative ";
		memNoise = memNoise + "memory noise: " + Util.round(noiseLevel, 3);
		drawText(g,row,col,memNoise);
	}
	
	/**
	 * 
	 * @param g
	 * @param row
	 * @param col
	 */
	private void writeDecay(Graphics g, int row, int col)
	{
		String decay = "decay/step: "  + Util.round(estimator.decay, 2);
		drawText(g,row,col,decay);
	}
	
	/**
	 * 
	 * @param g
	 * @param row
	 * @param col
	 */
	private void writeStepsUsed(Graphics g, int row, int col)
	{
		String stepUsed = "Steps used: " + (int)estimator.stepsUsed;
		drawText(g,row,col,stepUsed);
	}



	//------------------------------------------------------------------------------------------------------------------------------------
	//
	//------------------------------------------------------------------------------------------------------------------------------------	
	

	
	private void drawText(Graphics g, int down, int across, String text)
	{
		int x = 20 + 150*across;
		int y = 20 + 20* down;
		g.drawString(text, x, y);
	}
	
	
	


}
