package display;

import java.awt.*;
import java.util.Vector;

import Jama.Matrix;
import mPack.Estimator;
import mPack.Motion;
import mPack.SFM_Model;
import mPack.SenseData;
import mPack.Shapes;
import mPack.Truth;
import utilPack.Util;



public class DrawArea extends Canvas{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private IFrame frame;
	
	private SFM_Model model;
	
	private Truth truth;
	
	private SenseData sense;
	
	private Estimator estimator;
	
	private Motion motion;
 
	int maxObjects = SFM_Model.maxObjects;
	
	// values are 0,1,2 for left, centre and right areas
	private int actor;

	private int corner = 1;
	
	// type of object estimate to be drawn
	private int estType = 0;
	public void setEstType(int estType) {this.estType = estType;}

	
	// colours of object points
	private Color[] cols = {Color.BLUE, Color.RED, Color.GREEN, Color.CYAN, Color.YELLOW, Color.GRAY, Color.ORANGE, Color.PINK, Color.WHITE};
	private Color cols(int c)
	{
		int cp = c - 9*(c/9);
		return cols[cp];
	}
	
	
	private static String[] colour = {"blue","red","green","cyan","yellow","gray","orange","pink","white"};
	public static String colour(int c)
	{
		int cp = c - 9*(c/9);
		return colour[cp];
	}
	

	// radius of circle for drawing object points
	private int objRadius = 8;
		
	// radius of circle for drawing estimated object positions
	private int estRadius = 4;

	// radius of circle for drawing the end of the animal trajectory
	private int anRadius = 6;
	
	// radius for drawing a small blob at the end of each step
	private int blobRadius = 3;
	
	
	// suppress all lines of sight or circles of echo-location, after a mouse click on the animal
	private boolean suppressLines = false;
	
	// animal's drawn position, to detect mouse click on it
	private int xAnimal;
	private int yAnimal;
	
	// the current selected shape
	public int shape = -1;
			

	
	
	// fixed object points not rotated by viewing angle omega
	public double[][] unRotObjPoint = new double[SFM_Model.maxObjects][3];
	
	//fixed object points rotated by viewing angle omega
	public double[][] rotObjPoint = new double[SFM_Model.maxObjects][3];

	
	// track of the first object in the moving shape, not rotated
	public double[][] unRotObj0Track = new double[SFM_Model.maxSteps][3];
	
	// track of first object in the moving shape, rotated by phi
	public double[][] rotObj0Track = new double[SFM_Model.maxSteps][3];


	

	
	// animal trajectory rotated by angle omega
	public double[][] rotTrajPoint = new double[SFM_Model.maxSteps][3];
	
	// animal trajectory not rotated
	public double[][] unRotTrajPoint = new double[SFM_Model.maxSteps][3];
	

	

	// rotated lines of sight from the bee (latest position) to objects
	private double[][] rotLine = new double[SFM_Model.maxObjects][3];
	
	// unrotated lines of sight from the bee (latest position) to objects
	private double[][] unRotLine = new double[SFM_Model.maxObjects][3];
	


	// number of arcs to draw in bat echo-location circles	
	private int arcs = 50;
	
	// arcs of circles (unrotated)  to draw for bat echo-location data, with doppler shift
	private Matrix [][] unRotArc = new Matrix [SFM_Model.maxObjects][arcs];
		
	// arcs of circles (rotated by viewing angle omega)  to draw for bat echo-location data, with doppler shift
	private Matrix [][] rotArc = new Matrix [SFM_Model.maxObjects][arcs];

	

	// rotated estimate of the positions of objects at the latest step; point [x0, y0, z0]
	private double[][] rotEstimate = new double[SFM_Model.maxObjects][3];
	
	// unrotated estimate of the positions of objects at the latest step; point [x0, y0, z0]
	private double[][] unRotEstimate = new double[SFM_Model.maxObjects][3];
	

	
	// 3 rotated error bars from each central position estimate - displacements in [x,y,z] of one end of each error bar from [x0,y0,z0]
	// indices are [obj][error bar][x,y,z]
	private double [][][] rotErrorBar = new double[SFM_Model.maxObjects][3][3];
	
	// 3 unrotated error bars from each central position estimate - displacements in [x,y,z] of one end of each error bar from [x0,y0,z0]
	private Matrix [] unrotErrorBar = new Matrix[SFM_Model.maxObjects];
	
	

	// the selected object in this window
	private int selectedObject = -1;

	// number of object points to draw
	private int drawObjects = 0;

	// number of points along the trajectory to join by lines
	private int drawTrajPoints = 0;
	
	// latest step in the track
	private int latestStep = -1;
	
	// to ensure nothing will be drawn until this window is active	
	public void clear()
	{
		drawObjects = 0;
		drawTrajPoints = 0;
		latestStep = -1;
		passOnce = true;
	}
	
	// to stop the active window redrawing after a clear
	private boolean passOnce = false;
	
	// rotation angle, its cos and sin
	double omega;
	double sOmega;
	double cOmega;
	Matrix omegaRot = Matrix.identity(3, 3);

	// to draw error bars to both sides of a point estimate
	double[] sign = {1.0, -1.0};
	
	// true if this is the active window (only interrogate the model if thus is true)
	private boolean isActive() {return (actor == model.getWindow());}
	

    
    //--------------------------------------------------------------------------------------
    //                              Constructor
    //--------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param vf
	 * @param i
	 */
	public DrawArea(IFrame vf, SFM_Model model, Truth truth, SenseData sense, Estimator estimator,Motion motion, int i)
	{
		this.frame = vf;
		this.model = model;
		this.truth = truth;
		this.sense = sense;
		this.estimator = estimator;
		this.motion = motion;
		actor = i;
		setVisible(true);	
		
	    detectMouseMotion dMouseMotion = new detectMouseMotion();
	    addMouseListener(dMouseMotion); 
	    

	}
	
	//---------------------------------------------------------------------------------------
	//                                         drawing
	//---------------------------------------------------------------------------------------
	
	public void paint(Graphics g) 
	{
		String active = "inactive";
		if (isActive()) active = "active";
		 // Util.message("Draw area " + actor + ": " + active);
		 			 
		 setBackground(Color.WHITE);
		 int corner = 1;
		 int pixels = 511;

		 // clear out any previous drawing
		 g.setColor(Color.WHITE);
		 g.fillRect(corner,  corner,  pixels, pixels);
		 g.setColor(Color.BLACK);
		 g.drawRect(corner,  corner,  pixels, pixels);
		 
		 int displayType = model.getCaption(actor);
		 
		 if (displayType == SFM_Model.MOTION) drawMotionGraph(g);
		 
		 else if (displayType == SFM_Model.ACCURACY) drawErrorGraph(g);
				 
		 else if (displayType == SFM_Model.RANGEGRAPH) drawRangeGraph(g);
		 
		 else if (displayType == SFM_Model.SHOWHELP) showHelp(g);

		 else if (model.objectsStarted) 
		{
			 // dark background for left window only
			 if (model.isDark && (actor == 0)) 
			 {
				 g.setColor(Color.BLACK);
				 g.fillRect(corner,  corner,  pixels, pixels);
			 }
			 
			 // draw the animal, objects, estimates
			 drawScene(g, model.isDark);
		}
	}
	
	


	
	/**
	 * 
	 */
	private void drawScene(Graphics g, boolean isDark) 
	  {
		 if (isActive())
		 {
				// number of objects to draw
			 	drawObjects = truth.allPoints;

				// number of points along the trajectory to join by lines
				drawTrajPoints = truth.currentStep + 1;
				
				// latest step in the track
				latestStep = truth.currentStep;
		 }
		 
		// set angle of omega of view rotation about the x axis, in radians, and the rotation matrix
		 omegaRotate();
		 
		 if (model.controlledTest)
		 {
			 // draw Monte Carlo random likely points for controlled test
			 // drawLikelyPoints(g);			 
		 }
		 
		 if ((latestStep != -1) && !passOnce)
		 {
			 if (!isDark)
			 {
				 // draw all object points and the track of object 0, and tick marks for scale
				 drawObjectPoints(g);
				 
				 // draw the animal's trajectory
				 drawTrajectory(g);

				 // draw the latest sense data
				 if (model.species == model.BEE) drawLines(g);
				 if (model.species == model.BAT) drawCircles(g);
				 
			 }
			 // draw estimates of 3D positions, and possibly error bars
			 boolean drawErrorBars = !isDark;
			 drawEstimates(g,drawErrorBars,estType);
		}
		 
		 passOnce = false;		 
	  }
	
	/**
	 * set rotation parameters
	 */
	private Matrix omegaRotate()
	{
		omega= model.omega*Math.PI/90.0;
		
		// cos and sin
		cOmega = Math.cos(omega);
		sOmega = Math.sin(omega);
				
		// rotation matrix around x axis
		Matrix omegaRot = Matrix.identity(3, 3);
		omegaRot.set(1, 1, cOmega);
		omegaRot.set(1, 2, sOmega);
		omegaRot.set(2, 2, cOmega);
		omegaRot.set(2, 1, -sOmega);
		return omegaRot;
		
	}
	//-------------------------------------------------------------------------------
	//	                    Basic Drawing Methods             
	//-------------------------------------------------------------------------------
	
	
	//-------------------------------------------------------------------------------
	//	                    Draw coloured circles for object points           
	//-------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param g
	 */
	private void drawObjectPoints(Graphics g)
	{
		// draw tick marks for scale
		 drawTickMarks(g);
		 
		 
		// to reflect phi adjustment in the object points
		if (truth.madeObjects) placeObjects(truth.currentStep);
		
		// single colour for the regular shape, if drawn
		Color shapeColor = cols(truth.randomPoints);

		// draw all objects
		for (int p = 0; p < drawObjects; p++)
		{
			int x = (int)(256.0*rotObjPoint[p][0]) + corner;
			int y = (int)(256.0*rotObjPoint[p][1]) + 256 + corner;
			
			int rad = objRadius;
						
			g.setColor(cols(p));
			// for all points in a shape, a single colour and smaller circle
			if (p >= truth.randomPoints) 
			{
				g.setColor(shapeColor);
				rad = objRadius/2;
			}

			g.fillArc(x - rad, y-rad, 2*rad, 2*rad, 0, 360);
			g.setColor(Color.BLACK);
			g.drawArc(x - rad, y-rad, 2*rad, 2*rad, 0, 360);
		}
		
		// if there is any shape
		int verts = truth.allPoints - truth.randomPoints;
		if (verts > 0)
		{
			
			// draw the track of the first object in the moving shape
			g.setColor(Color.RED);
			int lastX = 0;
			int lastY = 0;				
			for (int st = 0; st < truth.currentStep + 1; st++)
			{
				int x = (int)(256.0*rotObj0Track [st][0]) + corner;
				int y = (int)(256.0*rotObj0Track [st][1]) + 256 + corner;
				
				if (st > 0)
				{
					g.fillArc(x - blobRadius, y-blobRadius, 2*blobRadius, 2*blobRadius, 0, 360);
					g.drawLine(lastX, lastY,  x,  y);
					
				}
				lastX = x;
				lastY = y;						
			}
			g.setColor(Color.BLACK);
		}
	}
	
	/**
	 * place object points to reflect the latest viewing angle omega
	 */
	private void placeObjects(int step)
	{
		
		
		if (isActive())
		{

			// compute unrotated object points for all boject
			for (int obj = 0; obj < drawObjects; obj++)
			{			
				unRotObjPoint[obj][0] = truth.objectLocation[step][obj].get(0, 0);
				double y = truth.objectLocation[step][obj].get(1, 0);
				double z =  truth.objectLocation[step][obj].get(2, 0);
				unRotObjPoint[obj][1] = y;
				unRotObjPoint[obj][2] = z;
			}
			
			// compute the track of the first object  of the moving shape, if there is one
			if (truth.allPoints  > truth.randomPoints)
			{
			    int movingObject = truth.randomPoints;
				for (int st = 0; st < step +1; st++)
				{
					Matrix firstObjPos = truth.objectLocation[st][movingObject];
					unRotObj0Track[st][0] = firstObjPos.get(0, 0);
					double y = firstObjPos.get(1, 0);
					double z =  firstObjPos.get(2, 0);
					unRotObj0Track[st][1] = y;
					unRotObj0Track[st][2] = z;
					
				}
			}
		}
						
		
		// rotate them
		rotateObjects(step);
		
	}

	
	/**
	 * 
	 */
	private void rotateObjects(int step)
	{
		//rotate all fixed objects
		for (int p = 0; p < drawObjects; p++)
		{			
			rotObjPoint[p][0] = unRotObjPoint[p][0];
			double y = unRotObjPoint[p][1];
			double z = unRotObjPoint[p][2];
			rotObjPoint[p][1] = y*cOmega + z*sOmega;
			rotObjPoint[p][2] = -y*sOmega + z*cOmega;
		}
		
		// rotate the red track of the first object of the moving shape, if there is one
		if (truth.allPoints > truth.randomPoints)
		{
			for (int st = 0; st < step + 1; st++)
			{
				rotObj0Track[st][0] = unRotObj0Track[st][0];
				double y = unRotObj0Track[st][1];
				double z = unRotObj0Track[st][2];
				rotObj0Track[st][1] = y*cOmega + z*sOmega;
				rotObj0Track[st][2] = -y*sOmega + z*cOmega;
			}
		}

	}
	
	/**
	 * 
	 * @param g
	 */
	private void drawTickMarks(Graphics g)
	{
		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial",Font.PLAIN,12));

		for (int tick = 1; tick < 10; tick++)
		{
			String val = tickText(tick);

			// x ticks
			int x = tick*256/5;
			g.drawLine(x, 511, x, 505);
			g.drawString(val, x-10, 500);

			// y axis ticks
			int y = 512 - tick*256/5;
			g.drawLine(0, y, 5, y);
			g.drawString(val, 10, y + 5);
		}
	}
	
	private String tickText(int tick)
	{
		String val = "0." + Integer.valueOf(2*tick).toString();
		if (tick > 4) val = "1." + Integer.valueOf(2*(tick-5)).toString();
		return val;
	}
	
	
	//-------------------------------------------------------------------------------
	//	                    Draw the animal's trajectory             
	//-------------------------------------------------------------------------------
	
	
	/**
	 * 
	 * @param g
	 */
	private void drawTrajectory(Graphics g)
	{
		g.setColor(Color.BLACK);

		// to reflect omega adjustment in the trajectory points
		if (truth.madeObjects) placeTrajectory();

		int xLast = 0;
		int yLast = 0;
		for (int p = 0; p < drawTrajPoints; p++)
		{
			int x = (int)(256.0*rotTrajPoint[p][0]) + corner;
			int y = (int)(256.0*rotTrajPoint[p][1]) + 256 + corner;
			
			// lines for the trajectory
			if (p > 0) 
			{
				g.setColor(Color.GREEN);
				g.drawLine(xLast, yLast, x, y);
				
				// small black circle for previous positions
				g.setColor(Color.BLACK);
				g.fillArc(x - blobRadius, y-blobRadius, 2*blobRadius, 2*blobRadius, 0, 360);
			}

			// larger black circle for the animal's final position
			if (p == drawTrajPoints - 1)
			{
				   g.fillArc(x - anRadius, y-anRadius, 2*anRadius, 2*anRadius, 0, 360);
				   
				   // so you can detect mouse clicks in the animal circel
				   xAnimal = x;
				   yAnimal = y;
			}
			
			xLast = x;
			yLast = y;
		}
	}

	
	/**
	 * place trajectory points unrotated
	 */
	private void placeTrajectory()
	{
		
		if (isActive())
		{

			// store unrotated trajectory points
			for (int p = 0; p < drawTrajPoints; p++)
			{
				for (int dim = 0; dim < 3; dim++)
					unRotTrajPoint[p][dim] = truth.animalPosition[p][dim];
			}
		}
 
		
		rotateTrajectory();
		
	}
	
	private void rotateTrajectory()
	{
		// compute rotated trajectory points
		for (int p = 0; p < drawTrajPoints; p++)
		{			
			rotTrajPoint[p][0] = unRotTrajPoint[p][0];
			double y = unRotTrajPoint[p][1];
			double z = unRotTrajPoint[p][2];
			rotTrajPoint[p][1] = y*cOmega + z*sOmega;
			rotTrajPoint[p][2] = -y*sOmega + z*cOmega;
		}
		
	}
	
	//-------------------------------------------------------------------------------
	//	                    Draw lines of sight (bee)             
	//-------------------------------------------------------------------------------
	
	
	/**
	 * 
	 */
	private void drawLines(Graphics g)
	{
		// suppress all lines of sight
		if (suppressLines) return;

		// no object selected; draw latest line for all objects
		if (selectedObject == -1)
			for (int obj = 0; obj < truth.allPoints; obj++) drawLine(g,latestStep, obj);
		
		// some object selected ; draw the last 5 lines of sight for that object
		else for (int st = 0; st < latestStep + 1; st++) 
			if (latestStep - st < 5) drawLine(g,st,selectedObject);
	}
	
	/**
	 * 
	 * @param g
	 * @param step
	 * @param obj
	 */
	private void drawLine(Graphics g, int step, int obj)
	{
		// to reflect adjustment of omega rotation in the trajectory points
		if (truth.madeObjects) placeLines(step,obj);
		
		int x = (int)(256.0*rotTrajPoint[step][0]) + corner;
		int y = (int)(256.0*rotTrajPoint[step][1]) + 256 + corner;
		
		// rotated unit vector to the object, 
		double rx = rotLine[obj][0];
		double ry = rotLine[obj][1];
		double rz = rotLine[obj][2];
		double d2 = (rx*rx + ry*ry + rz*rz);

		// scaled up by 5 to usually reach outside the drawing area
		double mult = 5.0;
		int dx = (int)(mult*256*rx);
		int dy = (int)(mult*256*ry);			
		g.drawLine(x,y,x + dx, y + dy);
	}
	
	
	
	/**
	 * calculate and store rotated unit vectors for bee lines of sight
	 */
	private void placeLines(int step, int obj)
	{
		
		if (isActive())
		{
			for (int dim = 0; dim < 3; dim++)
				unRotLine[obj][dim] = sense.beeVisionData[step][obj][dim];
		}
		
		
		rotateLines(step,obj);
		
	}
	
	private void rotateLines(int step, int obj)
	{
		rotLine[obj][0] =  unRotLine[obj][0];
		
		// rotate y and z components
		double y = unRotLine[obj][1];
		double z = unRotLine[obj][2];
		rotLine[obj][1] = y*cOmega + z*sOmega;
		rotLine[obj][2] = -y*sOmega + z*cOmega;
		
	}
	
	
	//-------------------------------------------------------------------------------
	//	         Draw circles from echo-location and doppler shift (bat)             
	//-------------------------------------------------------------------------------
	
	/**
	 * 
	 */
	private void drawCircles(Graphics g)
	{
		// suppress all circles
		if (suppressLines) return;

		// no object selected; draw latest circle for all objects
		if (selectedObject == -1)
			for (int obj = 0; obj < truth.allPoints; obj++) makeAndDrawCircle(g,latestStep, obj);
		
		// some object selected ; draw up to 3 circles for that object
		else for (int st = 0; st < latestStep + 1; st++) 
			if(latestStep - st < 3) makeAndDrawCircle(g,st,selectedObject);
	}
	
	
	/**
	 * 
	 * @param g
	 * @param step
	 * @param obj
	 */
	private void  makeAndDrawCircle(Graphics g, int step, int obj)
	{
		// if the unrotated circles need to be set
		if (isActive())
		{
			makeCircle(step,obj);
		}
		
		// compute omega rotated circle
		rotateCircle(obj);
		
		// draw it
		drawCircle(g,obj);


	}
	

	private void rotateCircle(int obj)
	{
		for (int arc = 0; arc < arcs; arc++)
		{
			rotArc[obj][arc] = omegaRotate().times(unRotArc[obj][arc]);
		}
	}
	
	/**
	 * 
	 */
	private void drawCircle(Graphics g,int obj)
	{
		int xFirst = 0;
		int yFirst = 0;
		int xLast = 0;
		int yLast = 0;
		g.setColor(Color.BLACK);

		for (int arc = 0; arc < arcs; arc++)
		{
			int x = (int)(256.0*rotArc[obj][arc].get(0, 0)) + corner;
			int y = (int)(256.0*rotArc[obj][arc].get(1, 0)) + 256 + corner;
			
			if (arc > 0) g.drawLine(xLast, yLast, x, y);
			else
			{
				xFirst = x;
				yFirst = y;
			}
			
			xLast = x;
			yLast = y;
		}
		
		g.drawLine(xLast, yLast, xFirst, yFirst);
		
	}
	
	/**
	 * 
	 * @param step
	 * @param obj
	 */
	private void makeCircle(int step, int obj)
	{
		// 5 -component data vector
		double[] dataVec = sense.batSonarData[step][obj];
		//current position of the bat
		Matrix batPos = Util.makeMatrix(truth.animalPosition[step]);
		
		// unit vector along the bat's velocity
		Matrix unit = Util.makeMatrix(dataVec);
		
		// radius of the echo-location sphere
		double d = dataVec[3];
		
		// angle inferred from doppler shift
		double cosTheta = dataVec[4];
		double sinTheta = Math.sqrt(1.0 - cosTheta*cosTheta);
		
		// radius of the circle
		double radius = d*sinTheta;
		
		// displacement of the centre of the circle from the bat, along the unit vector
		double cDist  = d*cosTheta;
		
		// centre of the circle
		Matrix centre = batPos.plus(unit.times(cDist));
		
		// make vectors on the circle
		for (int arc = 0; arc < arcs;arc++)
		{
			double phi = (2.0*Math.PI*arc)/arcs;
			double sinPhi = Math.sin(phi);
			double cosPhi = Math.cos(phi);
			
			// vector v is on any old circle
			Matrix v = new Matrix(3,1);
			v.set(0, 0, 0);
			v.set(1, 0, sinPhi);
			v.set(2, 0, cosPhi);
			
			// cross product with the unit vector
			Matrix w = Util.crossProduct(unit, v);
			
			// normalise w to the required radius
			double scale = radius/Math.sqrt(w.transpose().times(w).get(0, 0));
			w = w.times(scale);
			
			// add it to the centre
			unRotArc[obj][arc] = w.plus(centre);
		}
		
	}


	
	
	//------------------------------------------------------------------------------------------
	//	  Draw estimates of object positions and error bars, from the animal's latest position             
	//------------------------------------------------------------------------------------------
	
	/**
	 *  
	 * @param g
	 */
	private void drawEstimates(Graphics g, boolean drawErrorBars, int estType)
	 {
		
		// draw gray lines between the vertices of the shape
		int verts = truth.allPoints - truth.randomPoints;
		int step = truth.currentStep;
		// Util.message("Drawing estimates");
		
		// if there is a shape, draw gray lines between its estimated vertices if both vertices are moving
		if (verts > 0)
		{
			g.setColor(Color.GRAY);
			// try out all pairs of vertices, in either order (redundant but complete)
			for (int v1 = 0; v1 < verts; v1++)
			{
				 int p1 = v1 + truth.randomPoints;
				 boolean estimated1 = estimator.estimated[estType][step][p1];
				 
				 if (estimated1)
				 {
					 	placeEstimate(p1,estType,step,false);
						int x1 = (int)(256.0*rotEstimate[p1][0]) + corner;
						int y1 = (int)(256.0*rotEstimate[p1][1]) + 256 + corner;
						
						// is this point judged to be moving?
						boolean moving1 = motion.moving[p1][estType][step];

						for (int v2 = 0; v2 < verts; v2++)
						{

							int p2 = v2 + truth.randomPoints;
							boolean estimated2 = estimator.estimated[estType][step][p2];
							 
							 if (estimated2)
							 {
									placeEstimate(p2,estType,step,false);
									int x2 = (int)(256.0*rotEstimate[p2][0]) + corner;
									int y2 = (int)(256.0*rotEstimate[p2][1]) + 256 + corner;
																		
									// is this point judged to be moving?
									boolean moving2 = motion.moving[p1][estType][step];

									if (Shapes.hasEdge(shape,v1,v2))  g.drawLine(x1, y1, x2, y2);
							 }
						}
				 }
			}
		}

		
		
		
		for (int obj = 0; obj < drawObjects; obj++)
		 {
			 boolean estimated = estimator.estimated[estType][step][obj];
			 
			 if (estimated)
			 {
				 	placeEstimate(obj,estType,step,true);
				 	
					int x = (int)(256.0*rotEstimate[obj][0]) + corner;
					int y = (int)(256.0*rotEstimate[obj][1]) + 256 + corner;
								
					// draw thick gray error bars
					if (drawErrorBars)
					{
						// draw error bars out from it
						g.setColor(Color.lightGray);
						
						// three error bars - x,y projection
						for (int bar = 0; bar < 3; bar++)
						{
							int dx = (int)(rotErrorBar[obj][bar][0]*256);
							int dy = (int)(rotErrorBar[obj][bar][1]*256);
							// x goes left to right, y goes from top to bottom
							thickLine(g,4,x - dx, y - dy , x + dx, y + dy);						
						}
						
						g.setColor(Color.BLACK);
					}
					
					// draw a white  central estimate point, on top of the error bars
					g.setColor(Color.WHITE);
					// red if the points has been found to move
					if (motion.moving[obj][estType][step]) g.setColor(Color.RED);
					g.fillArc(x - estRadius, y-estRadius, 2*estRadius, 2*estRadius, 0, 360);
					g.setColor(Color.BLACK);
					g.drawArc(x - estRadius, y-estRadius, 2*estRadius, 2*estRadius, 0, 360);
					
			 }
		 }
	 }
	
	
	/**
	 * place and rotate estimates of the object position and error bars of anty estimate type
	 * @param obj
	 * 'param estType
	 */
	private void placeEstimate(int obj, int estType,int step,boolean write)
	{
		// Util.message("object " + obj + "; estimate " + estType);
		
		if (isActive())
		{
			// set the central estimate of position
			Matrix est = estimator.locationEstimate[estType][step][obj];
			
			// if (write) Util.message("Estimated object " + obj + " drawn at\t" + Util.mString(est));
			
			for (int d = 0; d < 3;d++)
				unRotEstimate[obj][d] = est.get(d, 0);
						
			// error bars, unrotated
			unrotErrorBar[obj] = estimator.locErrorBars[estType][step][obj];
			
		}

		
		rotateEstimate(obj);

	}

	/**
	 * 
	 * @param obj
	 */
	private void rotateEstimate(int obj)
	{
		// unrotated estimate
		double x  = unRotEstimate[obj][0];
		double y  = unRotEstimate[obj][1];
		double z  = unRotEstimate[obj][2];
				
		// apply the view rotation
		rotEstimate[obj][0] = x;
		rotEstimate[obj][1] = y*cOmega + z*sOmega;
		rotEstimate[obj][2] = -y*sOmega + z*cOmega;
		
		// error bars
		
		
		// columns of eBar are the 3 eigenvector directions
		Matrix eBar = unrotErrorBar[obj];

		// rotate the error bars by view angle omega and store them
		Matrix eRot = omegaRotate().times(eBar);

		// x,y,z
		for (int d = 0; d< 3; d++)
			// column/ error bar
			for (int bar = 0; bar < 3; bar++)
				rotErrorBar[obj][bar][d] = eRot.get(d, bar); // gets column bar from the Matrix
		
		// checkOrtho(obj);
	}
	
	
	/**
	 * check orthogonality of rotated error bars -
	 * off-diagonal elements should be zero
	 * 
	 * @param obj
	 */
	private void checkOrtho(int obj)
	{
		Util.message("\nError bar orthogonality check for object " + obj);
		for (int bar = 0; bar < 3; bar++)
		{
			String row = "";
			for (int car = 0; car < 3; car++)
			{
				// scalar product of two error bar vectors
				double dotp = 0.0;
				for (int d = 0; d< 3; d++)
					dotp = dotp + rotErrorBar[obj][bar][d]*rotErrorBar[obj][car][d]; 

				row = row + Util.round(dotp, 6) + "\t";
			}
			Util.message(row);			
		}
	}
	

	/**
	 * draw a thick line
	 * @param g
	 * @param thick
	 * @param x1
	 * @param y1
	 * @param x2
	 * @param y2
	 */
	private void thickLine(Graphics g,int thick, int x1, int y1, int x2, int y2)
	{
		int t2 = thick/2;
		for (int t = 0; t < thick; t++)
		{
			 int x1p = x1 - t2 + t;
			 int x2p = x2 - t2 + t;
			for (int u = 0; u < thick; u++)
			{
				 int y1p = y1 - t2 + u;
				 int y2p = y2 - t2 + u;
				 g.drawLine(x1p, y1p, x2p, y2p);
			}			
		}
	}
	
	//-------------------------------------------------------------------------------
	//	             Draw random points in region s of high likelihood            
	//-------------------------------------------------------------------------------

	/**
	 * 
	 * @param g
	 */
	private void drawLikelyPoints(Graphics g)
	{
		Vector<Matrix> likelyPoints = estimator.likelyPoints;
		
		int points = Math.min(400, likelyPoints.size());
		
		for (int p = 0; p < points; p++)
		{
			Matrix point = likelyPoints.get(p);
			Matrix rotPoint = omegaRotate().times(point);
			drawPoint(rotPoint,g);
		}
		
	}
	
	/**
	 * 
	 * @param rotPoint
	 * @param g
	 */
	private void drawPoint(Matrix rotPoint,Graphics g)
	{
		int x = (int)(256.0*rotPoint.get(0, 0)) + corner;
		int y = (int)(256.0*rotPoint.get(1, 0)) + 256 + corner;
					
		// draw the point as a short line
		g.setColor(Color.BLACK);
		g.drawLine(x, y, x, y);
		
	}
	
	//-------------------------------------------------------------------------------
	//	                 Graphs of Motion detection efficiency            
	//-------------------------------------------------------------------------------
	
	
	/**
	 * 
	 * @param g
	 */
	private void drawMotionGraph(Graphics g)
	{
		int xfirst = 80;
		int xLast = 480;
		int y0 = 400; // maximum success
		int y1 = 100; // zero success
		int ymid  = 250; // 50% success
		int xPerStep = 400/motion.trialSteps;
		int yPerSuccess = (y0 - y1)/100;
		g.setFont(new Font("Arial",Font.PLAIN,16));
		
		// number of points that are moving
		int maxMovers = truth.movingPoints;
		int nonMovers = truth.randomPoints;
		int allPoints = truth.allPoints;
		
		// explain points
		explainPoints(g);
		
		// gray lines for probabilities
		g.drawString("P=0.5",xfirst - 50,ymid +10);
		g.drawString("P=1.0",xfirst - 50,y1 + 10);
		g.setColor(Color.LIGHT_GRAY);
		g.drawLine(xfirst, ymid, xLast, ymid); // probability 0.5
		g.drawLine(xfirst, y1, xLast, y1); // probability 1.0

		// draw axes
		g.setColor(Color.BLACK);
		g.drawLine(xfirst, y0, xLast, y0);
		g.drawLine(xfirst, y0 + 1, xLast, y0 +1);
		g.drawString("Step", xLast - 30, y0 + 50);
		g.drawLine(xfirst, y0, xfirst, y1);
		g.drawLine(xfirst + 1, y0, xfirst + 1, y1);
		
		// ticks for steps, marked every 5 steps
		for (int step = 0; step < motion.trialSteps; step++)
		{
			int x = xfirst + step*xPerStep;
			int tick = 5;

			int mod5 = step - 5*(step/5);
			if (mod5 == 0) 
			{
				tick = 10;
				g.drawString(Integer.valueOf(step).toString(), x, y0 + 25);
			}
			g.drawLine(x, y0, x, y0 + tick);
		}
		
		// a green line below the x axis showing steps when object 0 moves
		g.setColor(Color.GREEN);
		int step1 = (int)truth.startStep;
		int step2 = (int)(truth.startStep + truth.inSteps);
		int x1 = xfirst  + step1*xPerStep;
		int x2 = xfirst  + step2*xPerStep;
		int yLine = y0 + 30;
		g.drawLine(x1, yLine, x2, yLine);
		g.drawLine(x1, yLine + 1, x2, yLine + 1);

		// captions showing start and end of motion
		g.setColor(Color.BLACK);
		g.drawString("Move", x1 - 20, y0 + 50);
		g.drawString("Stop", x2 - 20, y0 + 50);
		
		// colours to be plotted
		Color[] plotColours = {Color.BLACK,Color.RED};
		
		// first step to plot
		int firstStep = 2;
		
		// loop over estimation types (now types are tracking and noisy tracking)
		for (int t = 0; t < 2; t++)
		{
			g.setColor(plotColours[t]);
			
			int xPrev = x1;
			int yPrev = y1;
			for (int step = firstStep; step < motion.trialSteps; step++)
			{
				int x = xfirst + step*xPerStep;

				// mean percentage success over all trials
				int successes = motion.percentSuccess[step][t];
				int y = y0 - successes*yPerSuccess;

				// draw a square point
				g.fillRect(x, y -3, 6, 6);
				
				// draw a line from the previous point
				if (step > firstStep) g.drawLine(xPrev + 3, yPrev, x + 3, y);
				xPrev = x;
				yPrev = y;
				
			}
		}
	}

	
	//-------------------------------------------------------------------------------
	//	                 Graphs of RMS proportional depth errors            
	//-------------------------------------------------------------------------------
	

	
	/**
	 * draw graphs of RMS proportional relative errors,
	 * with and without memory noise
	 * @param g
	 */
	private void drawErrorGraph(Graphics g)
	{
		if (model.propError.length == 0) return;
		double[][] point = model.propError;

		g.setFont(new Font("Arial",Font.PLAIN,16));
		
		// find maximum steps and maximum mean error
		int maxStep = 0;
		double maxErr = 0.0;		
		for (int step = 0; step < point.length; step++) if (point[step][1] > 0.0)
		{
			maxStep = step;
			if (point[step][1] > maxErr) maxErr = point[step][1];
		}
		
		int xLeft = 80;
		int xAxis = 350;
		int yBot = 450;
		int yAxis = 300;
		int yTop = yBot - yAxis;
		
		// draw axes
		g.setColor(Color.BLACK);
		g.drawLine(xLeft, yBot, xLeft + xAxis, yBot);
		g.drawLine(xLeft, yBot, xLeft, yTop);
		g.drawString("Step", xLeft + xAxis + 5, yBot);
		g.drawString("Average ratio (error/range)", xLeft + 30, yTop - 30);
		
		// draw the maximum ratio r
		String text = "r=" + Util.round(maxErr, 2);
		g.drawLine(xLeft - 10, yTop, xLeft, yTop);
		g.drawString(text, xLeft - 50, yTop - 5);
		
		// draw up to 20 tick marks at distance 0.1 for the mean error ratio
		int yTick = (int)(0.1*yAxis/maxErr);
		if (yTick > 30) // if the tick marks are well spaced
		{
			for (int t = 1; t < 20; t++)
			{
				int yt = yBot - t*yTick;
				text = "r=0." + Integer.valueOf(t);
				// tick marks must be on the y axis, not above it
				if (yt > yTop + 50) 
				{
					g.drawLine(xLeft - 10, yt, xLeft, yt);
					g.drawString(text, xLeft - 50, yt - 5);
				}
			}
		}
		
		// draw step marks on the x axis
		for (int m = 1; m < 10; m++)
		{
			int step = 5*m;
			int xp = xLeft + (step*xAxis)/maxStep;
			if (xp < xLeft  + xAxis)
			{
				g.drawLine(xp, yBot,  xp, yBot + 10);
				String ts = Integer.valueOf(step).toString();
				g.drawString(ts, xp-5, yBot + 30);
			}
		}
		
		// explain points		
		explainPoints(g);
		
 		int xPrev = 0;
		int ynPrev = 0;
		int ytPrev = 0;
		for (int step = 0; step < maxStep; step++)
		{
			//scale point x to fit x axis
			int xp = xLeft + (step*xAxis)/maxStep;

			// scale point y to fit in maximum RMS error
			int yt = yBot -(int)(point[step][0]*yAxis/maxErr);
			int yn = yBot -(int)(point[step][1]*yAxis/maxErr);
			
			// draw noise RMS error for the step
			g.setColor(Color.RED);
			g.fillRect(xp, yn -6, 6, 6);
			// draw a line from the previous point
			if (step > 0) g.drawLine(xPrev + 3, ynPrev - 3, xp + 3, yn - 3);
			ynPrev = yn;

			// draw tracked RMS error for the step
			g.setColor(Color.BLACK);
			g.fillRect(xp, yt - 6, 6, 6);
			// draw a line from the previous point
			if (step > 0) g.drawLine(xPrev + 3, ytPrev - 3, xp + 3, yt - 3);
			ytPrev = yt;
			
			xPrev = xp;

		}
		
		// avoid an attempt to redraw the graph on Restart button
		model.setCaption(actor,SFM_Model.NONE);
	}
	
	/*
	 * 
	 		int xPrev = x1;
			int yPrev = y1;
			for (int step = firstStep; step < motion.trialSteps; step++)
			{
				int x = xfirst + step*xPerStep;

				// mean percentage success over all trials
				int successes = motion.percentSuccess[step][t];
				int y = y0 - successes*yPerSuccess;

				// draw a square point
				g.fillRect(x, y -3, 6, 6);
				
				// draw a line from the previous point
				if (step > firstStep) g.drawLine(xPrev + 3, yPrev, x + 3, y);
				xPrev = x;
				yPrev = y;
				
			}

	 */
	
	//-------------------------------------------------------------------------------
	//	         4 Graphs of likelihood versus range for the selected object           
	//-------------------------------------------------------------------------------
	
	
	/**
	 * 
	 * @param g
	 */
	private void drawRangeGraph(Graphics g)
	{
		int xLeft = 80;
		int xAxis = 400;
		int yBot = 450;
		int yAxis = 300;
		
		//colours for the 3 methods
		Color[] cols = {Color.BLACK,Color.GREEN,Color.RED};
		// titles
		String[] titles = {"Full Bayes","Tracking","Tracking with memory noise"};
		
		// loop over estimation  methods
		for (int meth = 0; meth < 3; meth++)
		{
			// text of explanation
			g.setColor(Color.black);
			int yTitle = 30 + 25*meth;
			g.drawString(titles[meth], 200, yTitle);
					
			g.setColor(cols[meth]);
			// coloured line of explanation
			g.drawLine(100,yTitle,180,yTitle);

			// pixel with the smallest negative log likelihood for this method
			int bestP = model.bestScan[meth];
			if (bestP > 0)
			{
				double bestNeg = model.rangeScan[meth][bestP];
				
				// draw the likelihood curve - scan 1 pixel at a time
				int previousY = yBot;
				for (int p = 0; p < 400; p++)
				{
					// negative log likelihood - best negative log likelihood (must be positive)
					double negGap = model.rangeScan[meth][p] - bestNeg;
					// likelihood/ best likelihood
					double likely = Math.exp(-negGap);
					
					int x = xLeft + p;
					int y = yBot - (int)(yAxis*likely);			
					g.drawLine(x-1, previousY, x, y);
					previousY = y;
				}
			}
		}

		// draw axes
		g.setColor(Color.LIGHT_GRAY);
		// light line for the maximum likelihood
		g.drawLine(xLeft, yBot - yAxis, xLeft + xAxis, yBot - yAxis);
		g.setColor(Color.BLACK);
		g.drawLine(xLeft, yBot, xLeft + xAxis, yBot); // x axis
		g.drawLine(xLeft, yBot, xLeft, yBot - yAxis); // y axis
		g.drawString("Range from animal to object", xLeft + xAxis - 150, yBot + 50);
		
		// draw distance ticks spaced by 0.2
		for (int d = 1; d < 8; d++)
		{
			double dist = 0.2*d;
			int xTick = xLeft + (int)(dist*256);
			g.drawLine(xTick, yBot, xTick, yBot + 10);
			g.drawString(tickText(d), xTick -10, yBot+ 25);
		}
		
		// draw a vertical line at the true range of the object
		int xTrue = xLeft + (int)(model.objectDistance*256);
		g.drawLine(xTrue, yBot, xTrue, yBot - 80);
		
		
	}
	
	/**
	 * Explain the points in the graph
	 * @param g
	 */
	private void explainPoints(Graphics g)
	{
		int xp = 100;
		int yp = 30;
		
		g.setFont(new Font("Arial",Font.PLAIN,16));

		g.setColor(Color.BLACK);
		g.drawString("Tracked", xp + 15, yp);
		g.fillRect(xp, yp - 6, 6, 6);
		
		yp = 60;
		g.drawString("Tracked with memory noise", xp + 15, yp);
		g.setColor(Color.RED);
		g.fillRect(xp, yp - 6, 6, 6);
		g.setColor(Color.BLACK);
		
	}
	
	//------------------------------------------------------------------------------------
	//                        Help
	//------------------------------------------------------------------------------------
	
	
	/**
	 * 
	 * @param g
	 */
	private void showHelp(Graphics g)
	{

		g.setFont(new Font("Arial",Font.BOLD,20));
		
		for (int lNo = 0; lNo < frame.helpText.length; lNo++)
		{
			String line = frame.helpText[lNo];
			if (line.startsWith("\b"))
			{
				g.setFont(new Font("Arial",Font.BOLD,14));
				line = line.substring(1);
				
			}
			g.drawString(line,20,40 + 16*lNo);
			g.setFont(new Font("Arial",Font.PLAIN,14));
		}

		
	}
	
	
	//-------------------------------------------------------------------------------
	//	                     Mouse movements and presses             
	//-------------------------------------------------------------------------------
	
	/* 
	 * rotated object placement when drawing:
	 * 
	 * int x = (int)(256.0*objPoint[p][0]) + corner;
	 * int y = (int)(256.0*objPoint[p][1]) + 256 + corner;
	 * 
	 * circle of radius objRadius
	*/

		
	  
	/**
	 *   
	 * detect a mouse click and do something
	 *
	 */
	class detectMouseMotion extends java.awt.event.MouseAdapter
	  {
	      public void mousePressed(java.awt.event.MouseEvent event)
	      {
	    	  // go back to no object selected, lines of sight not suppressed
	    	  selectedObject = -1;
	    	  suppressLines = false;
	    	  
	    	  // record where the press happened
	    	  int xp = event.getX();
	    	  int yp = event.getY();
	    	  
	    	  // in or out of the animal circle?
	    	  int xd = xAnimal - xp;
	    	  int yd = yAnimal - yp;
	    	  if ((xd*xd + yd*yd) < anRadius*anRadius) suppressLines = true;

	    	  // if (actor == 0)
	    	  {
		    	  for (int p = 0; p < (int)(truth.allPoints); p++)
		    	  {
		    			int x = (int)(256.0*rotObjPoint[p][0]) + corner;
		    			int y = (int)(256.0*rotObjPoint[p][1]) + 256 + corner;
		    			
		    			xd = x - xp;
		    			yd = y - yp;
		    			
		    			if ((xd*xd + yd*yd) < objRadius*objRadius) selectedObject = p;
		    	  }		    	  
		    	  
	    	  }	
	    	  
	    	  Util.message("Selected object " + selectedObject);
	    	  
	    	  model.selectedObject = selectedObject;
	    	  model.likelihoodGraph();
	    	  
	    	  repaint();
	      }
	  }
	


}
