package display;

public class Help {
	
	// Integers for topics
	public static int PROGRAM = 0;
	public static int GETTINGSTARTED = 1;
	public static int CONSEQUENCES = 2;
	public static int PAPERS = 3;

	public static int BEEBATMENU=4;
	public static int SHAPEMENU = 5;
	public static int NOISEMENU=6;

	public static int START = 7;
	public static int RESTART = 8;
	public static int STEP = 9;
	public static int ERRORS = 10;
	public static int MOTION = 11;
	public static int DARK = 12;
	public static int CHANGE_CALC = 13;

	public static int LEFT_SLIDERS = 14;
	public static int CENTRE_SLIDERS = 15;
	public static int RIGHT_SLIDERS = 16;

	public static int CLICK_OBJECT = 17;

	
	//----------------------------------------------------------------------------------
	//                          public data and methods
	//----------------------------------------------------------------------------------
	
	// submenu names
	public static String[]  subMenu = {"Introduction","Menus","Buttons","Sliders","Mouse Clicks"};
	
	// number of menu items per submenu (1 if the submenu name is a menu item)
	public static int[] itemCount = {4,3,7,3,1};
	
	// menu item names
	public static String[] topic = {"What The Program Does","Getting Started","Consequences","Related Papers",
			"Bee/bat menu","Moving Shapes Menu","Memory Error menu",
			"Start button","Restart button","Step button","Run button","Motion button","Dark button","Estimation type buttons",
			"Left sliders","Centre sliders","Right Sliders",
			"Mouse Click"
			};
	
	
	/**
	 *  help text for each help menu item
	 * @param topic
	 * @return
	 */
	public static String[] getHelp (int topic)
	{
		if (topic == PROGRAM) return intro;
		else if (topic == GETTINGSTARTED) return getStarted;
		else if (topic == CONSEQUENCES) return consequences;
		else if (topic == PAPERS) return papers;

		else if (topic == BEEBATMENU) return beeBat;
		else if (topic == NOISEMENU) return noise;
		else if (topic == SHAPEMENU) return shape;

		else if (topic == START) return start;
		else if (topic == RESTART) return restart;
		else if (topic == STEP) return step;
		else if (topic == ERRORS) return errors;
		else if (topic == MOTION) return motion;
		else if (topic == DARK) return dark;
		else if (topic == CHANGE_CALC) return calcType;

		else if (topic == LEFT_SLIDERS) return lSlide;
		else if (topic == CENTRE_SLIDERS) return cSlide;
		else if (topic == RIGHT_SLIDERS) return rSlide;

		else if (topic == CLICK_OBJECT) return clickObj;

		else return notWritten;
	}
	
	//----------------------------------------------------------------------------------
	//                            help text
	//----------------------------------------------------------------------------------
	
	
	private static String[] intro = {"What the program does","",
			"This program is a model of 3-D spatial cognition in bees and bats.",
			"It shows how animals compute an internal model of local 3-D space,",
			"which they use to control all their physical movements.",
			"",
			"\bThe Program",
			"A bee or a bat moves in a cubic space, with static points in it, shown as",
			"coloured circles, and moving shapes. It needs to know where these are",
			"in three dimensions, from sight or echo-location.",
			"It infers their 3-D positions as it moves, computing Shape From Motion.",
			"",
			"The program shows three ways in which animals can compute a 3-D model",
			"of the local space around them, and detect moving things: ",
			"(1) full Bayesian model (the best model, but expensive to compute)",
			"(2) tracking points in space, as the animal moves",
			"(3) tracking points, with random errors in short-term spatial memory.",
			"",
			"\bResults",
			"For a moving animal, the 3-D locations of things can be inferred",
			"from successive sightings; the estimated locations converge rapidly to",
			"the actual locations, in the Bayesian computation.",
			"",
			"Tracking is fast and builds a good model (almost as good as full Bayes)",
			"but it is sensitive to memory errors: a very small neural memory noise",
			"spoils the tracking model, and makes it useless for detecting motion."
			};
	
	private static String[] getStarted = {"Getting Started","",
			"\b(1) Press 'Start', then 'Run'.",
			"The bee moves across the space. As it moves, small circles with error bars",
			"show the estimated point positions, in its internal 3-D model of space.",
			"Lines of sight from the bee to the points are shown",
			"",
			"\b(2) Press 'Restart', then press 'Step' repeatedly.",
			"The bee moves along the same track, in slow motion. See how the estimates ",
			"converge to actual positions, and error bars get smaller.",
			"",
			"\b(3) Use the 'view rotate' slider.",
			"This shows the space from different angles, to see the third dimension.",
			"",
			"\b(4) Press 'Start', choose 'cube' from 'Moving Shapes', then press 'Run'.",
			"The corners of a cube appear. After a few steps, the cube starts to move. If",
			"the bee detects that the cube has moved, the corners of the cube turn red.",
			"",
			"\b(5) Choose 'Bat' from the 'Bee/Bat' menu, then repeat (1)",
			"Instead of lines of sight, circles from echo-location are shown.",
			"(these depend on echo delay and Doppler shift)",
			"",
			"\b(6) Click on the coloured circle for a point",
			"Recent lines of sight or echo circles to that point are shown",
			"",
			"\b(7) Click on the bee or bat",
			"All lines of sight or echo circles disappear.",
			"",
			"\b(8) Click the 'Noisy' button",
			"This shows the effect of memory errors on the accuracy of tracking"
			};
	
	private static String[] consequences = {"Consequences of the program","",
			"\b3-D Spatial Cognition",
			"3Spatial cognition is the first thing an animal brain does.",
			"All animals (and people) must understand the 3-D space around them,",
			"at all times, to make any movement. What animals do is move in space;",
			"what brains do is control their movement.",
			"",
			"\bA Gap in Neuroscience?",
			"For all the progress in neuroscience, there are no working neural models",
			"of capable 3-D spatial cognition. There in no working model of the first",
			"thing that brains do. This is a huge gap in our understanding of the brain.",
			"",
			"\bA reason for the Gap",
			"Why are there no working neural models of 3-D spatial cognition?",
			"In a neural model, 3-D positions are encoded as neural firing rates.",
			"These have errors. As shown by the program, typical neural memory error",
			"rates are too large to support a working model of 3-D space.",
			"That is why neural models have not been built.",
			"",
			"\bA Proposal ",
			"Spatial memory is stored in a wave in the brain, coupled to neurons.",
			"The wave is in the insect central body, or in the mammalian thalamus.",
			"This gives high precision and low spatial distortion. It can make 3-D",
			"spatial cognition work. The proposal is described in related papers."
			};
	
	
	private static String[] papers = {"Papers related to this work","",
			"\bThree-dimensional Spatial Cognition: Bees and Bats (BB.pdf)",
			"Describes this program, its results, and consequences.",
			"Even small levels of memory noise sharply degrade ",
			"the performance of the model, and of motion detection.",
			"This is a very hard challenge for neural models of spatial memory.",
			"An alternative is described, in which object locations",
			"are stored in a wave in the brain.",
			"",
			"\bThe Requirement for Cognition:",
			"\bDeriving a Requirement Equation (RE.pdf)",
			"Derives an equation for what brains need to do for greatest fitness.",
			"In complex domains this implies that brains build internal models of reality",
			"using Bayes theorem",
			"(such as the Full Bayesian model of this program)",
			"",
			"\bIs there a wave excitation in the thalamus?",
			"\b(https://arxiv.org/abs/2006.03420)",
			"Surveys evidence that the thalamus holds a wave excitation,",
			"storing 3-D spatial information",
			"",
			"\bSpatial Cognition: A Wave Hypothesis (WH.pdf)",
			"Surveys evidence that the central body of the insect brain",
			"and the mammalian thalamus hold a wave excitation",
			"whose role is to store short-term spatial memory",
			};
	
	private static String[] beeBat  = {"Bee/Bat Menu","",
		"Choose whether to run the model for a bee or a bat.",
		"Initially the model is set to run for a bee."
	};
	
	private static String[] shape = {"Moving Shapes Menu","",
			"After pressing the 'Start' button, use this menu to",
			"choose a regular shape to be placed at random",
			"amongst the static points.",
			"",
			"After some animal steps the shape will start to move, and the animal ",
			"tries to detect that it is moving.",
			"",
			"Use the 'Step' button to see if the animal detects motion",
			"or use the 'Motion' button to graph how well it does on average",
			"over 30 trials.",
			"",
			"Use sliders to choose:",
			" - the size of the shape",
			" - when it starts moving",
			" - how many steps it moves for",
			" - how far it moves"
			};
	
	private static String[] noise = {"Memory Error Menu","",
			"There are four types of random memory storage error:",
			"",
			"(1) 1-D: errors only along a line from the animal to the point",
			"(2) 3-D: errors in random directions",
			"(3) 1-D relative: errors only along lines to points",
			"(4) 3-D relative: errors in random directions ",
			"",
			"When errors are relative, nearby points have similar errors,",
			"with the overall error variance the same as for plain errors.",
			"This models a memory storing relative positions of nearby points,",
			"with random errors proportional to their separation.",
			"",
			"You can set the log of the level of these memory errors",
			"using the 'log memory noise' slider.",
			"",
			"Setting a value V on the slider gives an RMS memory error",
			"in storing the location of each point of 10**-V times the range" ,
			"from the animal to the point, whether or not the errors for ",
			"nearby points are relative.",
			"",
			"The selected type of memory storage error is shown at the left",
			"Memory errors can be varied independent of sense data errors."
	};
	
	private static String[] start = {"Start Button","",
			"Press this to stsrt a new random run of the model,",
			"or to test memory errors or motion detection.",
			"",
			"This will show the animal's start position, the random static points",
			"and moving shape (if any), and the initial lines of sight (bee)",
			"or circles from echo delay and doppler shift (bat).",
			"",
			"Set parameters by the sliders, such as:",
			"- Number of static points",
			"- Visual acuity of the bee, in degrees",
			"- Echo distance precision or doppler precision of the bat",	
			"- Step length (in x) in the animal's track",
			"- Exponential decay in log likelihoods for older steps"
	};
	
	private static String[] restart = {"Restart button","",
			"Press this to restart a run of the model or a test,",
			"with the same animal track, point and shape locations as the previous run.",
			"",
			"This will show the same random point positions as the previous run.",
			"",
			"Press the Restart button to re-enable the 'Errors' and 'Motion' buttons."
	};
	
	private static String[] step = {"Step button","",
			"Press this to take one step in the animal's track,",
			"and to see its new estimates of point positions.",
			"",
			"Three types estimate can be shown: ",
			"Full Bayesian, tracking, and tracking with memory noise.",
			"Select one of these types using the estimation type buttons."};
	
	private static String[] errors = {"Run button","",
			"Press this to run the model for 16 steps.",
			"At each step, the animal moves, and updates its estimates of the",
			"position of each point.",
			"There is then shown a graph of the proportional error in position,",
			"averaged over all points, as a function of step number,",
			"with and without memory errors.",
			"",
			"Proportional error is the distance from the estimate to the true position,",
			"divided by the range from the animal to the point,",
			"with a cutoff at small ranges .",
			"",
			"Two graphs are shown: black for tracking, red tracking with memory errors",
			"",
			"To explore the effect of memory errors, use sliders to vary:",
			"- the level of memory errors",
			"- the bee's visual acuity",
			"",
			"(Press 'Start' or 'Restart' to enable the 'Run' button)"
			};
	
	private static String[] motion = {"Motion button","",
			"Press this to run a test of motion detection.",
			"The selected shape moves linearly from a start step, for a number of steps.",
			"The shape moves towards the animal or away from it, ",
			"which makes motion detection depend on inference of ranges.",
			"",
			"The bee identifies the moving shape by",
			"allowing some set of points all to move by the same displacement",
			"and testing whether that significantly improves the Bayesian likelihood.",
			"",
			"A shape is detected if more than half of its vertices are moving points.",
			"(So there should be more static points than vertices of the shape,",
			" to reduce random spurious detection of motion)",
			"",
			"After 30 runs with different start positions for the static points and",
			"the moving shape (which take about a minute) you see a graph against ",
			"step number, of the probability that the animal detected the moving shape.",
			"Two graphs are shown: black for tracking, red for tracking with memory errors",
			"",
			"Use sliders to vary:",
			"- the level of memory errors",
			"- the number of static points (set this number large)",
			"- the bee's visual acuity",
			"- the distance the shape moves (as a proportion of its range from the bee)",
			"- the first step in which the shape moves",
			"- the number of steps for which it moves",
			"",
			"(Press 'Start' and choose a shape to enable the 'Motion' button)"
			};
	
	private static String[] dark = {"Dark button","",
			"Press this button to toggle the view between light and dark.",
			"In the dark view, only estimated object positions are shown as white dots,",
			" or red dots if movement has been detected.",
			"This can help see how estimated positions change with estimation method."};
	

	
	private static String[] lSlide = {"Left window sliders","",
			"\bview rotation",
			"Rotate the view about a horizontal axis to see the 3-D",
			"(this axis goes left to right across the middle of the window)",
			"",
			"\bstatic points",
			"The number of randomly placed static points, up to 40",
			"",
			"\bx step",
			"The size of a step of the animal's track in the x direction (left to right)",
			"",
			"\bsteps used (Bayes)",
			"For full Bayesian estimation, the number of recent steps whose sense data",
			"are combined to estimate 3-D positions",
			"",
			"\bmotion distance",
			"The distance the moving shape moves towards the animal or away from it,",
			"as a proportion of its range from the animal when it starts to move."
			};
	
	private static String[] cSlide = {"Centre window sliders","",
			"\bstep jitter",
			"random jitter of the animal's steps in y and z, as a proportion of the x step",
			"",
			"\binitial y step",
			"For the first step of its track, the animal has this value of dy/dx.",
			"(y is up and down in the 0 degree view)",
			"",
			"\blikelihood decay",
			"To model a small probability that objects might move: in full Bayes, the log  ",
			"likelihood contributions of old steps are divided by this factor for each step.",
			"For tracking, the previous estimate log likelihoods are divided by this factor.",
			"",
			"\bfirst move step",
			"For motion detection, the first step in which the moving shape moves"
			};
	
	private static String[] rSlide = {"Right window sliders","",
			"\brange error (bat)",
			"Std. deviation of random error in echo range, as a proportion of the range",
			"",
			"\bdoppler error (bat)",
			"Std. deviation of random error in cos(theta), where theta is the angle",
			"between the bat's motion and a line from the bat to the object,",
			"as measured by Doppler shift of the object's echo",
			"",
			"\bvisual acuity (bee)",
			"Std. deviation of a random angle between the line of sight and the true",
			"direction of the point from the bee",
			"",
			"\blog memory noise",
			"For noisy tracking if the slider value is V, the random memory error in storing",
			"a position estimate  is 10**(-V) times the range from the bee to the object.",
			"Errors can be in all directions, or just in the direction to the animal,",
			", and can be relative or absolute, as defined by a menu choice.",
			"",
			"\bmotion steps",
			"For motion detection, the number of steps for which the the moving shape moves"			
			};
	
	private static String[] calcType = {"Estimation type buttons","",
			"By clicking one of the three buttons 'Bayes', 'Track' or 'Noisy'",
			"You can display the estimates of point position made by that method.",
			"",
			"Estimated point positions are small white circles",
			"with grey error bars in three dimensions,",
			" or small red circles if motion of the point has been detected"
			};
	
	private static String[] clickObj = {"Click the mouse in the left window","",
			"\bWhen you click on a point,",
			"lines of sight to other points in the current step will not be shown.",
			"",
			"Lines of sight from the bee to the point will be shown for the 5 ",
			"latest steps. Similarly for the bat, circles of echo-location of the point ",
			"are shown for all steps ",
			"",
			"The central window will show up to three graphs",
			"of likelihood versus range from the animal to the selected point,",
			"by three estimation methods: ",
			"(1) Full Bayes, (2) tracking, and (3) tracking with memory errors.",
			"The true range from the animal to the point is also shown.",
			"",
			"\bWhen you click inside the black circle for the animal",
			"no lines of sight or echo circles will be shown.",
			"",
			"\bWhen you click outside any point",
			"the display will revert to showing lines of sight or circles ",
			"for all points for the latest step."};
	
		
	private static String[] notWritten = {"Some help","Help not written yet"};
	
	

}
