package display;


import java.awt.Adjustable;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollBar;
import javax.swing.JTextField;

import mPack.Estimator;
import mPack.Motion;
import mPack.SFM_Model;
import mPack.SenseData;
import mPack.Shapes;
import mPack.Truth;
import utilPack.Util;

public class IFrame extends JFrame{
	
	private Truth truth;
	
	private SenseData sense;
	
	private Estimator estimator;
	
	private Reporter reporter;
	
	private Motion motion;
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	// set true when a species is chosen
	private boolean started = false;

	// top corner of window on screen
	private int left = 50;
	private int top = 50;
	
	int windowWidth = 2400;
	int windowHeight = 900;

	public int getDrawAreaWidth() {return drawAreaWidth;}
	int drawAreaWidth;
	
	int drawingAreaWidth  = 513;

	// depth of drawing  areas
	private int drawHeight = 513;
	
	// depth of caption areas
	int captionDepth = 80;
	
	// width of left-hand button area
	private int buttonAreaWidth = 200;
	
	// three drawing areas
	public DrawArea getDrawArea(int i)  {return drawArea[i];}
	protected DrawArea[] drawArea = new DrawArea[3];
			
	// three caption areas
	public CaptionArea getCaptionArea(int i)  {return captionArea[i];}
	protected CaptionArea[] captionArea = new CaptionArea[3];

	private JLabel speciesName = new JLabel("");
	private JLabel errorTypeName = new JLabel("Memory Error Type:");
	private JLabel errorTypeChosen = new JLabel("Dummy");
	
	private JLabel email = new JLabel("rpworden@me.com");
		
	
	// private JLabel testName = new JLabel("Test Run Name");
	
	// private JTextField testNameInput = new JTextField();
	
	Vector<String> testNames = new Vector<String>();

	private int sliders = 5;
	
		
	// slider labels in 5*3 array
	String[][] scrollName = {
			{"view rotation","step jitter","range error (bat)"},
			{"static points","initial y step","doppler error (bat)"},
			{"x step","likelihood decay","visual acuity (bee)"},
			{"Steps used (Bayes)","Shape size","log memory noise"},
			{"motion distance","first move step","motion steps"},
	};
		
	// minimum and maximum values for all sliders
	private double[][][] minMax =
		{   {{-90,90},{0,1},{0.01,0.3}} ,
			{{1,40},{-1,1},{0.01,0.3}} ,
			{{0,1},{1,2.5},{0.1,10}} ,
			{{1,10},{0.1,0.8},{4,0}},
			{{-0.7,0.7},{1,30},{0,10}}
		};
	
	JLabel[][] scrollNameLabel = new JLabel[sliders][3];
	
	JLabel[][] scrollValueLabel = new JLabel[sliders][3];
	
	JScrollBar[][] scrollBar = new JScrollBar[sliders][3];
	
	private SFM_Model model;
	
	private boolean newTestRun = false;
	
	public String[] helpText;
	
	
	//--------------------------------------------------------------------------------------------
	//                     Constructor and frame setup
	//--------------------------------------------------------------------------------------------

	public IFrame(SFM_Model model, Truth truth, SenseData sense, Estimator estimator, Reporter reporter, Motion  motion, boolean show) throws Exception
	{
		super("Shape From Motion: Bats and Bees");
		
		this.model = model;
		this.truth = truth;
		this.sense = sense;
		this.estimator = estimator;
		this.reporter = reporter;
		this.motion = motion;
		model.frame = this;

		// stop the application on close
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		getContentPane().setLayout(null);
	    setResizable(true);
	    			    		
		// make the 3 drawing areas and caption areas
		for (int i = 0; i < 3; i++)
		{
			drawArea[i] = new DrawArea(this,model,truth,sense,estimator,motion, i);
			captionArea[i] = new CaptionArea(this,model,truth,sense,estimator,motion, i);
		}
		
		// set the coordinates of everything
		frameSetup(show);
		
		// make menus
		makeMenus();
		
		// make buttons
		makeButtons();
		
    	// set all the sliders
    	setSliders();
    	
		// make the first menu selections
		setSpecies("Bee");
		setMemoryNoiseType("3-D");
		
			
	}
	
	
	/**
	 * 
	 */
	public void frameSetup(boolean show)
	{
	    // coordinates of the whole frame on the screen
	    setBounds(left, top, windowWidth, windowHeight); // 50,50,2000,800
	    
    	// set up the drawing areas and caption areas
    	for (int i = 0; i < 3; i++)
    	{
    		int stepAreaLeft = buttonAreaWidth + i*(drawingAreaWidth + 40);

    		drawArea[i].setBounds(stepAreaLeft,20,drawingAreaWidth,drawHeight);
    		getContentPane().add(drawArea[i]);
    		
    		captionArea[i].setBounds(stepAreaLeft,19 + drawHeight,drawingAreaWidth,captionDepth);
    		getContentPane().add(captionArea[i]);
 
    	}
    	
    	    	
	    // display
	    if (show)
	    {
		    setVisible(true);
		    validate();
	    }
	    
	}
	
	
	//-------------------------------------------------------------------------------
	//                    Populate menus
	//-------------------------------------------------------------------------------
	
	// Menu bar and menus
	JMenuBar qMBar = new JMenuBar();
	JMenu helpMenu = new JMenu("Help");
	JMenu SpeciesMenu = new JMenu("Bee/Bat");
	JMenu shapeMenu = new JMenu("Moving Shapes");
	JMenu noiseMenu = new JMenu("Memory Errors");
	//JMenu testRunMenu = new JMenu("Tests");
	
	/**
	 * Make menus
	 */
	private void makeMenus()
	  {
		qMBar.add(helpMenu);
		qMBar.add(SpeciesMenu);
		qMBar.add(shapeMenu);
		qMBar.add(noiseMenu);
		
		makeSpeciesMenuItem("Bat", SpeciesMenu);
		makeSpeciesMenuItem("Bee", SpeciesMenu);
		
		makeNoiseMenuItem("1-D",noiseMenu);
		makeNoiseMenuItem("3-D",noiseMenu);
		makeNoiseMenuItem("1-D relative",noiseMenu);
		makeNoiseMenuItem("3-D relative",noiseMenu);

		makeHelpMenu(helpMenu);
		
		makeShapeMenu(shapeMenu);
		shapeMenu.setEnabled(false);
						
		setJMenuBar(qMBar);		    
	  }
	

	/**
	 * 
	 * @param name
	 * @param menuNo
	 * @param men
	 */
	private void makeSpeciesMenuItem(String name, JMenu men)
	{
		
		JMenuItem named = new JMenuItem(name);
		men.add(named);
						
		final String fName = name;

		named.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent ev) {
	        	setSpecies(fName);
	        }
	    });

	}
	
	private void makeNoiseMenuItem(String name, JMenu men)
	{
		
		JMenuItem named = new JMenuItem(name);
		men.add(named);
						
		final String fName = name;

		named.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent ev) {
	        	setMemoryNoiseType(fName);
	        }
	    });
	}
	
	/**
	 * set up the Help menu
	 * @param helpMenu
	 */
	private void makeHelpMenu(JMenu helpMenu)
	{
		JMenuItem[] helpItem = new JMenuItem[Help.topic.length + Help.subMenu.length];
		
		int index = 0;
		for (int sm = 0; sm < Help.subMenu.length; sm++)
		{
			int items = Help.itemCount[sm];
			String subName = Help.subMenu[sm];
			if (items == 1) 
			{
				addHelpItem(helpMenu, helpItem,index,subName);
				index++;
			}
			else
			{
				JMenu subMenu = new JMenu(subName);
				helpMenu.add(subMenu);
				Util.message("added submenu " + subName);
				for (int i = 0; i < items; i++)
				{
					String itemName= Help.topic[index];
					addHelpItem(subMenu,helpItem,index,itemName);
					index++;
				}
			}
		}
	}
	
	/**
	 * 
	 * @param menu
	 * @param helpItem
	 * @param index
	 * @param itemName
	 */
	private void addHelpItem(JMenu menu, JMenuItem[] helpItem , int index, String itemName)
	{
		final int top = index;
		helpItem[index] = new JMenuItem(itemName);
		
		helpItem[index].addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent ev) {
	        	showHelp(top);
	        }
	    });
		menu.add(helpItem[index]);
		
		// Util.message("added item " + index + " " + itemName);
	}
	

	/**
	 * set up the Help menu
	 * @param shapeMenu
	 */
	private void makeShapeMenu(JMenu shapeMenu)
	{
		JMenu[] dimension = new JMenu[2];
		dimension[0] = new JMenu("2-D");
		shapeMenu.add(dimension[0]);
		dimension[1] = new JMenu("3-D");
		shapeMenu.add(dimension[1]);
		int dim = 0;
		
		JMenuItem[] shapeItem = new JMenuItem[Shapes.shapeNames.length];
		for (int s = 0; s < Shapes.shapeNames.length; s++)
		{
			final int shape = s;
			shapeItem[s] = new JMenuItem(Shapes.shapeNames[s]);
			
			shapeItem[s].addActionListener(new ActionListener() {
		        public void actionPerformed(ActionEvent ev) {
		        	doShape(shape);
		        }
		    });
			
			if (s == Shapes.twoDShapes.length) dim = 1;
			dimension[dim].add(shapeItem[s]);			
		}
	}
	
	
	//----------------------------------------------------------------------------------------------
	//                   action methods for menu items
	//----------------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param topic
	 */
	private void showHelp(int topic)
	{
		helpText = Help.getHelp(topic);
		for (int l = 0; l < helpText.length;l++)
			Util.message(helpText[l]);
		model.setCaption(1, SFM_Model.SHOWHELP);
		model.doDrawingAndCaption(1);
	}
	

	/**
	 * On selecting Bat or Bee
	 * @param name
	 */
	private void setSpecies(String name)
	{
		Util.message("\nSelect " + name);
		started = true;
		setSliderStartValues();
		speciesName.setText(name);
		
		if (name.equals("Bat")) 
		{
			model.species = SFM_Model.BAT;
			model.setCaption(0, SFM_Model.BAT);
		}
		if (name.equals("Bee"))
		{
			model.species = SFM_Model.BEE;
			model.setCaption(0, SFM_Model.BEE);
		}
		
		startButton.setEnabled(true);
		disableOtherActions();
	}
	
	/**
	 * selecting a shape for motion detection
	 * @param sNo
	 */
	private void doShape(int sNo)
	{
		// Util.message("Selected shape " + Shapes.shapeNames[sNo]);
		
		// place the shape at random and record it
		truth.placeShape(sNo);
		
		// tell the left hand drawing area
		drawArea[0].shape = sNo;
		
		//enable motion detection plots
		motionButton.setEnabled(true);
		
		// show the tracking view
		doTypeButtonAction(1);
	}

	
	/**
	 * 
	 * @param fName
	 */
	private void setMemoryNoiseType(String fName)
	{
		errorTypeChosen.setText(fName);
		
		if (fName.startsWith("1-D")) estimator.noiseDimension = 1;
		else if (fName.startsWith("3-D")) estimator.noiseDimension = 3;
		estimator.relative = fName.length() > 4;
	}
	

	//-------------------------------------------------------------------------------
	//                        Make Buttons
	//-------------------------------------------------------------------------------
	
	
	// Main Buttons

	JButton startButton = new JButton("Start");
	JButton restartButton = new JButton("Restart");
	JButton stepButton = new JButton("Step");
	JButton runButton = new JButton("Run");
	JButton motionButton = new JButton("Motion");
	JButton darkButton = new JButton("Dark");
	
	/*
	JButton recordButton = new JButton("Record");
	JButton replayButton = new JButton("Replay");
	JButton saveButton = new JButton("Save");
	*/
	
	int buttonTop = 30;
	int buttonLeft = 30;
	int buttonSpacing = 60;
	int buttonWidth = 100;
	int buttonHeight = 30;
	
	// buttons to select which estimates are displayed		
	JButton fullButton = new JButton("Bayes");
	JButton trackButton = new JButton("Track");
	JButton noiseButton = new JButton("Noisy");
	
	
	/**
	 * 
	 */
	private void makeButtons()
	{		
		JButton[] mainButtons = new JButton[6];
		
		mainButtons[0] = startButton;
		mainButtons[1] = restartButton;
		mainButtons[2] = stepButton;
		mainButtons[3] = runButton;
		mainButtons[4] = motionButton;
		mainButtons[5] = darkButton;
		
		
		for (int b = 0; b < mainButtons.length; b++)
		{
			mainButtons[b].setSize(buttonWidth, buttonHeight);
			mainButtons[b].setBounds(buttonLeft, buttonTop + b*buttonSpacing, buttonWidth, buttonHeight);
			final int bb = b;
			
			mainButtons[b].addActionListener(new ActionListener() {
		    	final int buttonId = bb;
		        public void actionPerformed(ActionEvent ev) {
		        	doMainButtonAction(buttonId);
		        }
		    });
			
			getContentPane().add(mainButtons[b]);
			mainButtons[b].setVisible(true);
			mainButtons[b].setEnabled(false);;
		}
		
		
		// add a species label below all main buttons
		int labelTop = buttonTop + (mainButtons.length + 1)*buttonSpacing;
		speciesName.setBounds(buttonLeft, labelTop - 80, 120,40);
		speciesName.setFont(new Font("TimesRoman", Font.BOLD, 20));
		getContentPane().add(speciesName);
		
		// add a fixed memory error type label 
		int typeLabelTop = labelTop  - 70 + buttonSpacing;
		errorTypeName.setBounds(buttonLeft, typeLabelTop, 140,40);
		errorTypeName.setFont(new Font("TimesRoman", Font.BOLD, 14));
		getContentPane().add(errorTypeName);
				
		// add a dynamic memory error type chosen label
		typeLabelTop = typeLabelTop + 20;
		errorTypeChosen.setBounds(buttonLeft, typeLabelTop, 140,40);
		errorTypeChosen.setFont(new Font("TimesRoman", Font.BOLD, 14));
		getContentPane().add(errorTypeChosen);

		// email label
		labelTop = labelTop - 100;
		email.setBounds(buttonLeft, 780, 140,40);
		email.setFont(new Font("TimesRoman", Font.BOLD, 16));
		getContentPane().add(email);


		JButton[] typeButtons = new JButton[3];
		
		typeButtons[0] = fullButton;
		typeButtons[1] = trackButton;
		typeButtons[2] = noiseButton;
		
		// grid of display type buttons
		for (int b = 0 ; b < 3; b++)
		{
			typeButtons[b].setSize(100,30);
			typeButtons[b].setBounds(buttonLeft, buttonTop + 8*buttonSpacing +  30*b, 100, 30);
			final int bc = b;
			
			
			typeButtons[b].addActionListener(new ActionListener() {
		    	final int buttonIdx = bc;
		        public void actionPerformed(ActionEvent ev) {
		        	doTypeButtonAction(buttonIdx);
		        }
		    });
		    
			
			getContentPane().add(typeButtons[b]);
			typeButtons[b].setFont(new Font("Arial",Font.PLAIN,12));
			typeButtons[b].setVisible(true);
		}
		
		
	}
	
	//----------------------------------------------------------------------------------------------
	//                   action methods for buttons
	//----------------------------------------------------------------------------------------------
	

	/**
	 * button actions
	 * @param buttonId
	 */
	void doMainButtonAction(int buttonId)
	{
		try
		{
			
			// start
			if (buttonId == 0)
			{
				// cycle the window to draw on - starting from 0 for any test case, or for a new random run
				 if (newTestRun) 
				 {
					 model.setWindow(0);
					 newTestRun = false;
				 }
				 // subsequent starts of the same test run - not random starts
				 else model.setWindow(model.getWindow() + 1);// 0 1  or 2
				 
				model.setWindow(0);

				model.start(true);

				restartButton.setEnabled(true);
				stepButton.setEnabled(true);
				runButton.setEnabled(true);
				motionButton.setEnabled(false);
				darkButton.setEnabled(true);
				shapeMenu.setEnabled(true);
				
			}
			
			// restart
			else if (buttonId == 1)
			{
				model.restart(true);

				runButton.setEnabled(true);
				darkButton.setEnabled(true);
			}

			
			// step
			else if (buttonId == 2)
			{
				// step and redraw
				model.step(true);
				
				runButton.setEnabled(false);
				motionButton.setEnabled(false);
				shapeMenu.setEnabled(false);
			}
			
			// plot errors
			else if (buttonId == 3)
			{
				model.plotErrors();
				runButton.setEnabled(false);
			}
			
			
			// motion detection
			else if (buttonId == 4)
			{
				// not implemented yet
				motion.detectMotion();
				motionButton.setEnabled(false);
				runButton.setEnabled(false);
			}
			
			// toggle dark view
			else if (buttonId == 5)
			{
				model.isDark = !model.isDark;
				model.doDrawingAndCaption(0);
			}

		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	

		
	/**
	 * change the type of object estimate type displayed
	 * @param b
	 */
	private void doTypeButtonAction(int b)
	{
		int[] estType = {estimator.FULLBAYES,estimator.TRACK,estimator.NOISYTRACK};
		estimator.estTypeDrawn = estType[b];
		
		for (int window = 0; window < 3; window++)
		{
			drawArea[window].setEstType(estType[b]);
			drawArea[window].repaint();
			captionArea[window].repaint();
		}
	}

	//----------------------------------------------------------------------------------------------
	//                                      Sliders
	//----------------------------------------------------------------------------------------------
	
	/**
	 * 
	 */
	private void setSliders()
	{
    	// all sliders and their labels
    	for (int win = 0; win < 3; win++)
        	for (int dep = 0; dep < sliders; dep++)
        	{
       		
        	    scrollValueLabel[dep][win] = new JLabel("0"); // to avoid a null pointer later       		
        		
        		// compute (x,y) coordinates
        		int left = buttonAreaWidth + win*drawingAreaWidth;
        		int down = 40 + drawHeight + captionDepth + dep*32;
        		
        		// add a slider and its listener in the array
        		scrollBar[dep][win] = new JScrollBar(Adjustable.HORIZONTAL);
        	    scrollBar[dep][win].setBounds(left + 100, down, drawingAreaWidth - 200, 30);
       	    
        		final int fwin = win;
        		final int fdep = dep;
        		AdjustmentListener scrollListener = new AdjustmentListener() {
        	        public void adjustmentValueChanged(AdjustmentEvent adjEvent) 
        	        {
        	        	doScrollAdjust(fwin,fdep,adjEvent.getValue());
        	        }
        	      };
        	      scrollBar[dep][win].addAdjustmentListener(scrollListener);
        	      scrollBar[dep][win].setValue(45); // set to centre; scale is 0..90
        	    add(scrollBar[dep][win]);
        	    
        	    // add a label in the array
        	    // maximum 17 characters make 120 pixels; 7 pixels/char
        	    scrollNameLabel[dep][win] = new JLabel(scrollName[dep][win]);
        	    int width = scrollName[dep][win].length()*7;
        	    scrollNameLabel[dep][win].setBounds(left + 100 - width, down, width, 30);
        	    scrollNameLabel[dep][win].setAlignmentX(RIGHT_ALIGNMENT); // has no effect
        	    add(scrollNameLabel[dep][win]);
        	    
        	    // labels for values of sliders
        	    String defValue = "";
        	    if (win != 0) defValue = "0";
        	    scrollValueLabel[dep][win] = new JLabel(defValue); // repeat
        	    scrollValueLabel[dep][win].setBounds(left + drawingAreaWidth - 90, down, 60, 30);
        	    add(scrollValueLabel[dep][win]);
        	}
		
	}
	
	/**
	 * convert from model values to scroll values in the range 0..90
	 * @param modelVal
	 * @param win
	 * @param dep
	 * @return
	 */
	private int makeScrollVal(int win, int dep, double modelVal)
	{
		double minVal = minMax [dep][win][0];
		double maxVal = minMax [dep][win][1];
		
		int scroll = (int)(90.0*(modelVal - minVal)/(maxVal - minVal));
		return scroll;
	}
	
	/**
	 * set a slider and its value label
	 * @param win
	 * @param dep
	 * @param modelVal
	 */
	private void setScroll(int win, int dep, double modelVal)
	{
		scrollBar[dep][win].setValue(makeScrollVal(win,dep, modelVal));
		scrollValueLabel[dep][win].setText(Double.valueOf(modelVal).toString());
	}
	
	/**
	 * convert from scroll values in the range 0..90 to model values
	 * @param scrollVal
	 * @param win
	 * @param dep
	 * @return
	 */
	private double makeModelVal(int win, int dep, int scrollVal)
	{
		double minVal = minMax [dep][win][0];
		double maxVal = minMax [dep][win][1];
		
		double mVal  = minVal + ((double)scrollVal)*(maxVal - minVal)/90.0;
		return mVal;		
	}
	
	
	/**
	 * set start values of all sliders from the model
	 * (set each slider and its value label)
	 */
	private void setSliderStartValues()
	{
		setScroll(0,0,model.omega);
		setScroll(0,1,truth.randomPoints);
		setScroll(0,2,truth.stepSize);
		setScroll(0,3,estimator.stepsUsed);
		setScroll(0,4,truth.motion);
		
		setScroll(1,0,truth.jitter);
		setScroll(1,1,truth.ySpeed);
		setScroll(1,2,estimator.decay);
		setScroll(1,3,truth.shapeSize);
		setScroll(1,4,truth.startStep);
		
		setScroll(2,0,sense.range_err);
		setScroll(2,1,sense.doppler_err);
		setScroll(2,2,sense.getErrBee());
		setScroll(2,3,estimator.mem_noise);
		setScroll(2,4,truth.inSteps);
	}

	
	
	/**
	 * actions for sliders
	 * @param group 0,1,2 for left, centre, right window
	 * @param dep depth 0..3
	 * @param scroll a scroll value in the range 0..90
	 */
	private void doScrollAdjust(int group, int dep, int scroll)
	{
		
		// do nothing until a species is selected
		if (!started) return;
		
		double modelVal = makeModelVal(group,dep, scroll);
		double rounded = (int)(100.0*modelVal)/100.0; // 2 decimal places

		// display slider value
		scrollValueLabel[dep][group].setText(Double.valueOf(rounded).toString());
		
		boolean phiChange = false;
		
		// left slider group
		if (group == 0) 
		{
			// for the phi slider, repaint all windows
			if (dep == 0) 
			{
				phiChange = true;
				model.omega = modelVal;
				for (int window = 0; window < 3; window++)
				{
					drawArea[window].repaint();
					captionArea[window].repaint();
				}
			}
			// change the number of objects
			if (dep == 1) 
			{
				truth.randomPoints = (int)modelVal;
				// do not draw, or allow most actions, until after the start button is pressed, with the new number of objects
				model.objectsStarted = false;
				disableOtherActions();
				
			}
			if (dep == 2) truth.stepSize = modelVal;
			if (dep == 3) estimator.stepsUsed = modelVal;
			if (dep == 4) truth.motion = modelVal;
		}
		
		// centre slider group
		if (group == 1) 
		{
			if (dep == 0) truth.jitter= modelVal;
			if (dep == 1) truth.ySpeed = modelVal;
			if (dep == 2) estimator.decay = modelVal;
			if (dep == 3) truth.shapeSize = modelVal;
			if (dep == 4) truth.startStep = modelVal;
		}
		
		
		// right slider group
		if (group == 2) 
		{
			if (dep == 0) sense.range_err= modelVal;
			if (dep == 1) sense.doppler_err= modelVal;
			if (dep == 2) sense.setErrBee(modelVal);
			if (dep == 3) estimator.mem_noise = modelVal;
			if (dep == 4) truth.inSteps = modelVal;
		}
		
		// for slider changes other than phi, only repaint the active window
		if (!phiChange && (model.getWindow() != -1)) drawArea[model.getWindow()].repaint();

	}
	
	private void disableOtherActions()
	{
		restartButton.setEnabled(false);
		stepButton.setEnabled(false);
		runButton.setEnabled(false);
		motionButton.setEnabled(false);
		
	}

	
	/**
	 * 
	 */
	

}
