package mPack;

import display.IFrame;
import display.Reporter;
import utilPack.Util;

public class MainClass {
	
	private IFrame frame;
	public IFrame getFrame() {return frame;}
	
	/**
	 * 
	 */
	public MainClass()
	{
		try
		{
			// class for ground truth
			Truth truth  = new Truth();			
			
			//class for sense data
			SenseData sense = new SenseData(truth);
			
			// class for making estimates
			Estimator estimator = new Estimator(truth,sense);
			
			Motion motion = new Motion(truth,sense,estimator);
			
			// class for test reporting
			Reporter reporter = new Reporter(truth, sense, estimator, motion);
					
			
			// make the SFM model
			SFM_Model model = new SFM_Model(truth, sense,estimator, reporter, motion);
			
			// connect them
			truth.setModel(model);
			sense.setModel(model);
			estimator.setModel(model);
			motion.setModel(model);
			reporter.setModel(model);
			estimator.setrReporter(reporter);
			
		    // make the user window true = show it
			frame = new IFrame(model, truth, sense,estimator, reporter,motion, true);
		    		    
		    // show the frame
			frame.validate();
		    
		}
		catch (Exception ex) {Util.message("Exception starting: " + ex.getMessage()); ex.printStackTrace();}
	}

	
	
	  //------------------------------------------------------------------------------------------------
	  //                              main method
	  //------------------------------------------------------------------------------------------------

	  public static void main(String[] args) {
	    new MainClass();
	  }

}
