package mPack;

import display.IFrame;
import display.Reporter;
import utilPack.Util;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Hashtable;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import Jama.Matrix;

/**
 * shape from motion computation
 */

public class SFM_Model {
	
	public IFrame frame;
	
	private Truth truth;
	
	private SenseData sense;
	
	private Estimator estimator;
	
	private Reporter reporter;
	
	private Motion motion;
	
	
	
	public int species = 0;
	
	// allowed species
	public static int BAT = 1;
	public static int BEE = 2;
	
	// maximum number of steps in track
	public static  int maxSteps = 100;
	
	// maximum number of objects
	public static int maxObjects = 60;
	
	
	// true for dark view
	public boolean isDark = false;
	
	// false if no noise is to be added to sense data, for  test
	boolean addNoise = true;
	
	// temporary for controlled test of Monte Carlo
	public boolean controlledTest = false;
	
	// to record displacements
	// private Vector<Matrix> displacements = new Vector<Matrix>();
	

	// what is displayed in each window
	public int getCaption(int win) {return captions[win];}
	public void setCaption(int win, int cType) {captions[win] = cType;}
	private int[] captions = {0,0,0};
	
	// allowed values of the window displays and their captions
	public static int NONE = 0;
	// public static int BAT = 1; (reuse)
	// public static int BEE = 2;  (reuse)
	public static int ACCURACY = 3;
	public static int MOTION = 4;
	public static int RANGEGRAPH = 5;
	public static int SHOWHELP = 6;
	
	
	
	/**
	 *  starter values of sliders
	 */

	// yz rotation in degrees
	public double omega = 0.0;

	
	
	
	
	
	public int selectedObject = -1;
	
	
	
	// chi squared test of the model estimates and errors against the true object position;
	double chiSquared = 0.0;
	
	// true if the start button has been pressed since the boject number slider wasslid
	public boolean objectsStarted = false;

	//--------------------------------------------------------------------------------------
    //              Dynamical model data  - information content and chi squared
    //--------------------------------------------------------------------------------------
	
	
	// information content of sense data at each step
	public double[] senseDataInformation = new double[maxSteps];
	
	// cumulative information in sense data
	public double senseInformationCumulative = 0.0;
	
	// information content (bits) of 3-D model at each step
	public double[] modelInformation = new double[maxSteps];
	
	// chi squared fit of the model to the real object positions at each step
	public double[] modelChiSquared = new double[maxSteps];
	
	// number of degrees of freedom for chi squared
	public double[] degreesOfFreedom = new double[maxSteps];
	
    //--------------------------------------------------------------------------------------
    //                             Test runs and windows
    //--------------------------------------------------------------------------------------
		
	
	// test runs in the Test menu
	private Hashtable<String,TestRun> testRuns = new Hashtable<String,TestRun>();
	
	// the test run currently selected
	public TestRun selectedTestRun;
	
	// the window 0, 1 or 2 to draw on; will be cycled by the start button
	public void setWindow(int win) 
	{
		// Util.message("setting window to " + win);
		drawWindow = win;
		if (drawWindow > 2) drawWindow = 0;
	}
	
	// the active window
	public int getWindow() {return drawWindow;}
	private int drawWindow = -1;
	
	public void doDrawingAndCaption(int win)
	{
		frame.getCaptionArea(win).repaint();
		frame.getDrawArea(win).repaint();
	}
	
    //--------------------------------------------------------------------------------------
    //                          Recording and statistics
    //--------------------------------------------------------------------------------------
	
	private StepData[] allStepData = new StepData[maxSteps];
	
	public double[][] propError = new double[maxSteps][2];
	
	public int plotStepErrors = 2;
		
    //--------------------------------------------------------------------------------------
    //                              Constructor
    //--------------------------------------------------------------------------------------
	
	
	public SFM_Model(Truth truth, SenseData sense, Estimator estimator, Reporter reporter, Motion motion)
	{
		this.truth = truth;
		this.sense = sense;
		this.estimator = estimator;
		this.reporter = reporter;
		this.motion = motion;
	}
	   
    //--------------------------------------------------------------------------------------
    //                              Button Methods
    //--------------------------------------------------------------------------------------
	
	
	/**
	 * start button
	 */
	public void start(boolean draw)
	{
		// display 3-D view in the left hand window
		setCaption(0,species);
		
		String mess = "Start ";
		// Util.message(mess);
		
		
		// set all randomised animal positions and velocities
		truth.makeAllAnimalSteps();
		
		// undo any previous object selection
		selectedObject = -1;
		
		// make a new set of objects at random positions
		truth.allPoints = truth.randomPoints;
		truth.makeObjects();
		objectsStarted  = true;
		
		// undo any previously selected shape
		truth.isShape = false;
		
		
		//  start the action
		restart(draw);
		
	}
	
	/**
	 * restart button
	 */
	public void restart(boolean draw)
	{
		setCaption(2,NONE);

		// start at step zero
		truth.currentStep = 0;
		
		// empty all step data
		emptyStepData();
		
		// zero information scores
		zeroInformation();
		
		// gather data for the zeroth step
		gatherSenseData();
				
		// clear random points in high likelihood regions
		estimator.likelyPoints = new Vector<Matrix>();
		
		// mark no points as moving, but do not re-zero the counts
		motion.startMotionCounts(false);
		
		// draw objects and animal at the current phi in the left window
		if (draw)
		{
			doDrawingAndCaption(0);
			setCaption(2,NONE);
			doDrawingAndCaption(2);
		}

	}
	
	private void emptyStepData()
	{
		for (int step = 0; step < maxSteps; step++)
		{
			allStepData[step] = new StepData();
			propError[step][0] = 0.0;
			propError[step][1] = 0.0;
		}
	}

	
	
	/**
	 * 
	 */
	public void step(boolean draw)
	{
		setCaption(2,NONE);
		doDrawingAndCaption(2);
		
		// increment the step number
		truth.currentStep++;
		int step = truth.currentStep;
				
		String mess = "\nStep " + step;
		Util.message(mess);
				
		// gather animal's sense data for the step - new randoms
		gatherSenseData();
		
		// clear random points in high likelihood regions
		estimator.likelyPoints = new Vector<Matrix>();
		
		// update estimates of object positions
		estimator.estimatePositions(step);
				
		// record step data for this step
		StepData sd = new StepData(this,truth,sense,estimator,species,step);
		allStepData[step] = sd;
		
		// if an object is selected, draw its range likelihood graph
		if (selectedObject > -1) likelihoodGraph();
		
		// detect motion of any shape shown
		motion.detectMotion(step);

		
		// write error comparison
		// sd.writeErrorCompare(true);
				

		// draw objects and animal trajectory at the current phi in the left window
		if (draw)
		{
			doDrawingAndCaption(0);
		}
		
	}
	
	// true distance of animal from selected object
	public double objectDistance;
	
	// negative log likelihoods along a pixel scan from the animal through the selected object, by 3 estimation methods
	public double[][] rangeScan = new double[3][400];
	
	// range from the animal in pixels which has the smallest negative log likelihood for the selected object, by 3 estimation methods
	public int[] bestScan = new int[3];
	
	/**
	 * plot range likelihood graphs for the selected object
	 * @param obj
	 */
	public void likelihoodGraph()
	{
		int obj = selectedObject;
		int step = truth.currentStep;
		
		if (obj > -1)
		{
			//type  of display in the centre window
			setCaption(2,RANGEGRAPH);
			
			// three estimation methods to plot - full Bayes, run track, and run noisy track
			int[] estMethod = {estimator.FULLBAYES ,estimator.TRACK ,estimator.NOISYTRACK};
			// three calculation methods
			int[] cMethod = {estimator.RUN_BAYES ,estimator.RUN_TRACK ,estimator.RUN_NOISY_TRACK };
			
			// initialise variables used for plotting
			for (int meth =0; meth < 3; meth++)
			{
				for (int g = 0; g < 400;g++) rangeScan[meth][g] = -1.0;
				bestScan[meth] = -1;
			}
			
			// real object position
			Matrix trueObj = truth.objectLocation[step][obj];
			
			// animal position
			Matrix animalPos = Util.makeMatrix(truth.animalPosition[step]);
			
			// displacement from animal to object
			Matrix disp = trueObj.minus(animalPos);
			
			// distance from animal to object
			double dist = Math.sqrt(disp.transpose().times(disp).get(0, 0));
			objectDistance = dist;
			
			// unit vector from the animal to the object
			Matrix unit = disp.times(1.0/dist);
			
			// loop over methods
			for (int meth = 0; meth < 3; meth++)
			{
				int estType = estMethod[meth];
				int calcMethod = cMethod[meth];
				double bestBayes = 1000.0;

				if (estimator.estimated[estType][step][obj]) try
				{
					// steps of 1 pixel in x
					for (int p = 0; p < 400;p++)
					{
						// 512 pixels make a distance 2.0
						double pixDist = ((double)p)/256.0;

						// position whose likelihood is to be computed
						Matrix ePos = animalPos.plus(unit.times(pixDist));
						
						double trueNegLog = estimator.negLogLikelihood(obj,makeVec(ePos),null,calcMethod);	
						rangeScan[meth][p] = trueNegLog;
						if (trueNegLog < bestBayes) 
						{
							bestBayes = trueNegLog;	
							bestScan[meth] = p;
						}
						
					}
				}
				catch (Exception ex) {bestScan[meth] = -2;} // catch any calculation problem silently
			}
			
			Util.message("Best scan pixels " + bestScan[0] + ";" + bestScan[1] + ";" + bestScan[2]);
			
		}
		else
		{
			setCaption(2,NONE);
		}
		
		doDrawingAndCaption(2);
	}
	
	private Timer timer = null;


	
	
	/**
	 * run from start and plot RMS distance errors vs. step
	 */
	public void plotErrors()
	{
		int steps = (int)(1.8/truth.stepSize);
		if (steps > 40) steps = 40;

		TimerTask stepTask = new TimerTask() {
		        public void run() {
		        	if (truth.currentStep < 16) 
		        	{
		        		step(true);
		        	}
		        	else finishUp();
		        }
		};

		timer = new Timer();
		timer.schedule(stepTask, 600,600); 
		
	}
	
	/**
	 * 
	 */
	private void finishUp()
	{
		Util.message("Finishing run");
		timer.cancel();
		
		// draw final state
		doDrawingAndCaption(0);
		
		// summarise relative errors, with and without noise
		if (species == BEE) writeRelativeErrors(truth.currentStep);
	}
	
	
	//-----------------------------------------------------------------------------------------------
	//
	//-----------------------------------------------------------------------------------------------	
	
	/**
	 * 
	 * @param steps
	 */
	private void writeRelativeErrors(int steps)
	{
		Util.message("\nMean proportional depth errors");
		Util.message("\nStep\tTracked\tNoisy\tRatio");
		for (int step = 0; step < steps;step++)
		{
			StepData sd = allStepData[step];

			double[] re = sd.writeMeanErrorCompare(false);
			double ratio = 0.0;
			if (re[1] > 0.0) ratio = re[1]/re[0];
			propError[step][0] = re[0];
			propError[step][1] = re[1];

			Util.message(step + "\t" + Util.round(re[0],4) + "\t" + Util.round(re[1],4) + "\t" + Util.round(ratio,4));
		}
		
		setCaption(2,ACCURACY);
		doDrawingAndCaption(2);
	}
	
	
	/**
	 * 
	 */
	public void record(String name)
	{
		// Util.message("Record");
		TestRun testRun = makeTestRun(name);
		testRuns.put(name,testRun);
	}
	
	/**
	 * 
	 */
	public void replay()
	{
		// Util.message("Special test");
	}
	
	/**
	 * re-purposed for special test
	 */
	public void save()
	{
		Util.message("Controlled Test");
		try
		{
			estimator.doControlledTest();
			controlledTest = true;
			
			// draw point inside one standard deviation
			frame.getDrawArea(0).repaint();
		}
		catch (Exception ex) {}
	}
	
	
	
	/**
	 * reset information counts
	 */
	private void zeroInformation()
	{
		for (int step = 0; step <maxSteps; step++)
		{
			senseDataInformation[step]= 0.0;
			modelInformation[step]= 0.0;
			modelChiSquared[step] = 0.0;
			degreesOfFreedom[step] = 0;
		}
		
		senseInformationCumulative = 0.0;
	}


	/**
	 * 
	 */
	private void gatherSenseData()
	{
		// loop over objects
		for (int obj = 0; obj < truth.allPoints; obj++)
		{
			if (species == BEE) sense.gatherBeeData(obj);
			
			else if (species == BAT) sense.gatherBatData(obj);
		}

		
	}
	
	
  
	 //--------------------------------------------------------------------------------------
	 //                              Vector and matrix stuff
	 //--------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param v
	 * @return
	 */
	public static double[] makeVec(Matrix v)
	{
		double[]vec = new double[3];
		if (v!= null) for (int dim = 0; dim <3; dim++) vec[dim] = v.get(dim,0);
		else
		{
			Util.message("Null vector matrix");
		}
		return vec;
	}
	

	
	//--------------------------------------------------------------------------------------------
	//                                          Test runs
	//--------------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @return
	 */
	public double[] setParam()
	{
		double[] param = new double[12];

		param[0] = omega;
		param[1] = truth.stepSize;
		param[2] = truth.jitter;
		param[3] = truth.ySpeed;
		param[4] = sense.range_err;
		param[5] = sense.doppler_err;
		param[6] = sense.getErrBee();
		param[7] = estimator.decay;
		param[8] = estimator.threshold;
		
		return param;
	}

	/**
	 * 
	 * @param param
	 */
	public void getParam(double[] param)
	{ 

		omega = param[0];
		truth.stepSize = param[1];
		truth.jitter = param[2];
		truth.ySpeed = param[3];
		sense.range_err = param[4];
		sense.doppler_err = param[5];
		sense.setErrBee(param[6]);
		estimator.decay = param[7];
		estimator.threshold = param[8];
		
	}

	/**
	 * 
	 * @return
	 */
	public TestRun makeTestRun(String name)
	{
		return new TestRun(name, species, setParam(), truth.objectLocation, truth.animalPosition, truth.currentStep);
	}
	
	/**
	 * 
	 * @param ts
	 */
	public void getTestRun(TestRun ts)
	{
		species = ts.beeBat;
		getParam(ts.parameter);
		// truth.objectLocation = ts.object;
		truth.animalPosition = ts.pos;
		truth.currentStep = ts.steps;
	}
	

}
