package mPack;

import utilPack.Util;

public class SenseData {
	
	private Truth truth;
	
	private SFM_Model model;
	
	
	// standard deviation of error in range (echo location)
	public  double range_err = 0.06;
		
	// standard deviation of doppler error in range (echo-location)
	public  double doppler_err = 0.06;
		
	// standard deviation of error in vision angle in degrees
	public double getErrBee() {return err_bee;}
	public void setErrBee(double err_bee) 
	{
		this.err_bee = err_bee;
		// Util.message("Set err_bee to " + err_bee);
	}
	public  double err_bee = 0.8;


	
	
    //--------------------------------------------------------------------------------------
    //              Dynamical model data - animal's sense data
    //--------------------------------------------------------------------------------------
	
		
	// visual data of the bee; for each object at each step, a unit vector  in [x,y,z]
	public double[][][] beeVisionData = new double[SFM_Model.maxSteps][SFM_Model.maxObjects][3];
	
	// echo-location data for each step, each object
	// the last double [5] is [ux, uy, uz, range, doppler shift(cos theta) ] 
	// where u is the unit vector along the  bat's velocity
	public double[][][] batSonarData = new double[SFM_Model.maxSteps][SFM_Model.maxObjects][5];

		

	
	
	//----------------------------------------------------------------------------------------
	//
	//----------------------------------------------------------------------------------------
	

	
	public SenseData(Truth truth)
	{
		this.truth = truth;
		
	}
	
	public void setModel(SFM_Model model) {this.model = model;}
	
	
	//----------------------------------------------------------------------------------------
	//
	//----------------------------------------------------------------------------------------
	
	/**
	 * gather bee's sense data for the step
	 * Set up arrays (for all objects, current step):
	 * 
	 * sense.beeVisionData[truth.currentStep][obj][3]
	 * 
	 */
	public void gatherBeeData(int obj)
	{
		// common for bee and bat
		double[] disp = new double[3];
		
		//displacement of object from latest animal position
		double act2 = 0.0;
		for (int d = 0; d < 3; d++)
		{
			disp[d] = truth.objectLocation[truth.currentStep][obj].get(d, 0) - truth.animalPosition[truth.currentStep][d];
			act2 = act2 + disp[d]*disp[d];
		}
		// actual distance from the animal to the object
		double act = Math.sqrt(act2);
		double rd2 = act2;
					
		// calculate line of sight data
		if (model.addNoise)
		{
			// add proportional random Gaussian errors to vector components before unitising
			rd2 = 0.0;
			for (int d = 0; d <3; d++)
			{
				// err_bee is a standard deviation in degrees
				disp[d] = disp[d]*(1.0 + Util.randomGaussian()*getErrBee()*Math.PI/90.0);
				rd2 = rd2 + disp[d]*disp[d];
			}
		}

		double rd = Math.sqrt(rd2);				
		double dRatio = rd/act; // should be near 1.0
		int i = 1;

		// store unit direction vector for each object
		for (int d = 0; d <3; d++)
			beeVisionData[truth.currentStep][obj][d] = disp[d]/rd;
		
	}


	/**
	 * gather bat's sense data for the step
	 * Set up arrays (for all objects, current step):
	 * 
	 * sense.batSonarData[currentStep][obj][5]
	 * 
	 */
	public void gatherBatData(int obj)
	{
		
		//displacement of object from latest animal position
		double act2 = 0.0;
		double[] disp = new double[3];
		for (int d = 0; d < 3; d++)
		{
			disp[d] = truth.objectLocation[truth.currentStep][obj].get(d, 0) - truth.animalPosition[truth.currentStep][d];
			act2 = act2 + disp[d]*disp[d];
		}
		// actual distance from the animal to the object
		double act = Math.sqrt(act2);
		double rd2 = act2;
					
		// store an object range and doppler shift for a bat, with absolute errors (uses the bat velocity vector)		
		// calculate and store unit vector in direction of previous velocity
		double speed2 = 0.0;
		double[] unit = new double[3];
		for (int dim = 0; dim < 3; dim++) 
		{
			unit[dim] = truth.animalVelocity[truth.currentStep][dim];
			speed2  = speed2 + unit[dim]*unit[dim];
		}
		speed2 = Math.sqrt(speed2);
		
		// first three components of bat sonar data are the unit vecotr along its velocity
		for (int dim = 0; dim < 3; dim++)
		{
			unit[dim] = unit[dim]/speed2;
			batSonarData[truth.currentStep][obj][dim] = unit[dim];
		}
		
		// cosine of angle between unit vector and object displacement from bat
		double cosTheta = 0.0;
		for (int dim = 0; dim < 3; dim++)
			cosTheta = cosTheta + unit[dim]*disp[dim]/act;

		// random noise
		if (model.addNoise) cosTheta = cosTheta + doppler_err*Util.randomGaussian();
		batSonarData[truth.currentStep][obj][4] = cosTheta;
		
		// range from echo-location
		double dist = Math.sqrt(act2);
		// random noise
		if (model.addNoise) dist = dist + range_err*Util.randomGaussian();
		batSonarData[truth.currentStep][obj][3] = dist;
		
		boolean writeData = false;
		if (writeData)
		{
			Util.message("\nObject " + obj + "; step: " + truth.currentStep);
			Util.message("Object position:" + vText(model.makeVec(truth.objectLocation[truth.currentStep][obj])));
			Util.message("Bat position:" + vText(truth.animalPosition[truth.currentStep]));
			Util.message("Displacement :" + vText(disp));
			Util.message("Bat velocity:" + vText(truth.animalVelocity[truth.currentStep]));
			Util.message("Unit along velocity :" + vText(unit));
			Util.message("Doppler Cos theta: " + Util.round(cosTheta, 5));
			Util.message("Echo Range: " + Util.round(dist, 5));
		}
	}
	
	/**
	 * 
	 * @param vec
	 * @return neat text for a 3-vector array
	 */
	public  static String vText(double[]  vec)
	{
		String tex = "";
		for (int d = 0; d < 3; d++)
			tex  = tex + "\t" + Util.round(vec[d], 4);
		return tex;
	}

	


}
