package mPack;

import utilPack.Util;

/**
 * data for regular shapes which move
 */

public class Shapes {
	
	public static int LINE = 0;
	public static int TRI1 = 1;
	public static int TRI2 = 2;
	public static int SQUA = 3;
	public static int RECT = 4;
	public static int PARA = 5;
	public static int RHOM = 6;
	public static int HEXA = 7;
	public static int CUBE = 8;
	public static int BOX = 9;
	public static int TOBL = 10;
	public static int PILL = 11;
	public static int WEDG = 12;
	public static int PPPD = 13;
	
	public static String[] shapeNames = {"line","triangle1","triangle2","square","rectangle","parallelogram","rhombus","hexagon",
			"cube","box","toblerone","pill","wedge","parallelipiped"};
	
	
	// 2-D shapes
	static int[][] line = {{-1,0},{0,0},{1,0},{2,0}};
	static int[][] tri1 = {{-1,0},{1,0},{1,1}}; // right angled triangle
	static int[][] tri2 = {{-1,0},{1,0},{0,1}}; // isosceles triangle
	static int[][] squa = {{-1,-1},{-1,1},{1,1},{1,-1}};
	static int[][] rect = {{-2,-1},{-2,1},{2,1},{2,-1}};
	static int[][] para = {{-2,-1},{-2,1},{2,2},{2,0}};
	static int[][] rhom = {{-2,0},{0,-1},{2,0},{0,1}};
	static int[][] hexa = {{-3,0},{-2,-2},{0,-2},{1,0},{0,2},{-2,2}};
	
	public static int[][][] twoDShapes = {line,tri1,tri2,squa,rect,para,rhom,hexa};
	
	static int[] threeDSlices = {SQUA,SQUA,TRI2,HEXA,TRI1,PARA};
	

	
	
	/**
	 * 
	 * @param shape
	 * @return
	 */
	public static double[][] getPoints(int shape)
	{
		// all 2-D shapes
		if (shape < twoDShapes.length) return getPoints(shape, 0.0);
		
		// 3-D extrusions of 2D shapes		
		if (shape == CUBE) return merge(getPoints(SQUA,1.0),getPoints(SQUA,-1.0));
		
		if (shape == BOX) {return merge(getPoints(SQUA,3.0),getPoints(SQUA,-3.0));}
		
		if (shape == TOBL) {return merge(getPoints(TRI2,3.0),getPoints(TRI2,-3.0));}
		
		if (shape == PILL) {return merge(getPoints(HEXA,0.5),getPoints(HEXA,-0.5));}
		
		if (shape == WEDG) {return merge(getPoints(TRI1,1.0),getPoints(TRI1,-1.0));}
		
		if (shape == PPPD) {return merge(getPoints(PARA,1.0),getPoints(PARA,-1.0));}
		
		return getPoints(LINE);
	}
	
	
	
	/**
	 * 
	 * @param shape2D
	 * @param z
	 * @return
	 */
	private static double[][] getPoints(int shape2D, double z)
	{
		int[][] s2 = twoDShapes[shape2D];
		double [][] point = new double[s2.length][3];

		for (int p = 0; p < s2.length;p++)
		{
			int[] p2 = s2[p];
			point[p][0] = (double)(p2[0]);
			point[p][1] = (double)(p2[1]);
			point[p][2] = z;			
		}
		
		return point;		
	}
	
	
	/**
	 * 
	 * @param shape
	 * @param start
	 * @param end
	 * @return
	 */
	public static boolean hasEdge(int shape, int start, int end)
	{
		boolean hasEdge = false;
		
		if ((start >= 0)&& (end >=0) && (shape >= 0))
		{
			
			// cycling round edges of any 2D shape
			if (shape < twoDShapes.length)
			{
				int verts = twoDShapes[shape].length;
				if ((start < verts -1) && (end == start + 1)) hasEdge = true;
				if ((start == verts -1) && (end == 0)) hasEdge = true;
			}
			
			// 3D solids
			else if (shape < twoDShapes.length + threeDSlices.length)
			{
				// the 2D shape which is a slice of this 3D solid
				int slice = threeDSlices[shape - twoDShapes.length];
				
				// number of vertices of the 2D slice
				int verts = twoDShapes[slice].length;
				
				// one end of the solid
				if (hasEdge(slice,start,end)) hasEdge = true;
				
				// the other end of the solid
				if (hasEdge(slice,start - verts,end -verts)) hasEdge = true;
				
				// lines connecting the slice-like ends
				if ((start-verts) == end) hasEdge = true;
				if ((end-verts) == start) hasEdge = true;
			}
			
		}
		
		// if (shape >= twoDShapes.length) Util.message("shape " + shape + "; edge from " + start + " to " + end + "; " + hasEdge);
		
				
		return hasEdge;
	}
	
	/**
	 * 
	 * @param s1
	 * @param s2
	 * @return
	 */
	private static double[][] merge(double[][] s1,double[][] s2)
	{
		int len = s1.length + s2.length;
		double[][] res = new double[len][3];
		
		for (int p1 = 0; p1 < s1.length;p1++) res[p1] = s1[p1];
		for (int p2 = 0; p2 < s2.length;p2++) res[s1.length + p2] = s2[p2];
		
		return res;
	}
	


}
