package mPack;

import Jama.Matrix;
import utilPack.Util;

public class Truth {
	
	private SFM_Model model;
	
	
	// true when objects have been placed
	public boolean madeObjects = false;
		
	// number of steps taken so far
	public int currentStep = 0;
	
	// number of random fixed points in 3-D space
	public int allPoints;
	
	// total number of point, including a fixed shape
	public int randomPoints = 6;
	
	// length of each step dx taken by the animal from centre left
	public double stepSize = 0.1;
	
	// random variation of dy/dx and dz/dx for each step
	public double jitter = 0.6;
	
	// initial velocity dy/dx
	public double ySpeed = -0.3;
	
	
	// size of a  regular shape, added to the random objects
	public double shapeSize = 0.15;
	
	
	
	// largest movement of the object that moves (obj = 0), after all moving steps
	public double motion = 0.4;
	
	// first step it moves
	public double startStep = 6.0;

	// number of steps for which it moves
	public double inSteps = 3.0;
	
	// shapes and motion detection
	public boolean isShape = false;
	public int shapeSelected;
	public int movingPoints;
	public String shapeName;
	

	
    //--------------------------------------------------------------------------------------
    //              Dynamical model data - ground truth
    //--------------------------------------------------------------------------------------
	
	// real (x,y,z) coordinates of up to maxObjects objects for maxSteps steps (not rotated)
	public Matrix[][] objectLocation = new Matrix[SFM_Model.maxSteps][SFM_Model.maxObjects];
	
	
	// animal's (x,y,z) velocity before each step
	public double[][] animalVelocity = new double[SFM_Model.maxSteps][3];	
	
	// animal's position before each step
	public double[][] animalPosition = new double[SFM_Model.maxSteps][3];	
	
	//----------------------------------------------------------------------------------------
	//
	//----------------------------------------------------------------------------------------
	
	public Truth()
	{
		
	}
	
	public void setModel(SFM_Model model) {this.model = model;}
	
	
	
	//----------------------------------------------------------------------------------------
	//
	//----------------------------------------------------------------------------------------
	

	
	/**
	 * 
	 */
	public void startAnimalPosition()
	{
		currentStep = 0;
		
		// starting position
		animalPosition[0][0] = 0.1;
		animalPosition[0][1] = 0.0;
		animalPosition[0][2] = 0.0;
		
		// starting velocity - change in position per step, in units of stepSize
		animalVelocity[0][0] = 1.0;
		animalVelocity[0][1] = ySpeed;
		animalVelocity[0][2] = 0.0;
	}
	
	/**
	 * 
	 */
	public void makeAnimalStep(int step)
	{

		// move from the previous position with the previous velocity
		for (int d = 0; d < 3; d++) 
			animalPosition[step][d] = animalPosition[step -1][d] + stepSize*animalVelocity[step - 1][d];
		
		// make two random Gaussians, with standard deviation  = jitter in lateral velocity
		double dy = jitter*Util.randomGaussian();
		double dz = jitter*Util.randomGaussian();
		
		// stop the animal getting outside its tube
		double y = animalPosition[step-1][1];
		double z = 0.0;
		z = animalPosition[step-1][2];
		double radius = Math.sqrt(y*y + z*z);
		double damp = 1.0 -radius;
		
		// compute the current velocity (x velocity constant, damping and jitter in y and z)
		animalVelocity[step][0] = 1.0;
		animalVelocity[step][1] = damp*(animalVelocity[step - 1][1] + dy);
		animalVelocity[step][2] = damp*(animalVelocity[step - 1][2] + dz);
		
	}
	
	
	/**
	 * if edge = 0.1,
	 * make random objects in x from 0.1 to 1.9, and y*y + z*z < 0.9
	 */
	public void makeObjects()
	{
		int obj = 0;

		// edge of drawing area with no objects
		double edge = 0.1;
		
		// randomly place all objects, not moving
		while (obj < randomPoints)
		{
			double x = edge + (2.0 - edge)*Math.random();
			double y = 2.0*Math.random() - 1.0;
			double z = 2.0*Math.random() - 1.0;
			
			if ((y*y + z*z) < (1.0 - edge)*(1.0 - edge))
			{
				double[] pos = new double[3];
				pos[0] = x;
				pos[1] = y;
				pos[2] = z;
				
				// initially set the same position for all steps
				for (int step = 0; step < model.maxSteps; step++)
					objectLocation[step][obj] = Util.makeMatrix(pos);

				
				obj++;
			}
		}
		

		
		madeObjects = true;
	}
	
	/**
	 * add the vertices of a regular shape to the randomly placed objects, 
	 * scaled as defined by the shape size slider,
	 * rotated at random 
	 * and placed at random to be all inside the volume, 
	 * 
	 * @param shapeNo
	 */
	public void placeShape(int shapeNo)
	{
		isShape = true;
		shapeSelected = shapeNo;
		shapeName = Shapes.shapeNames[shapeNo];
		
		double[][] rawShape = Shapes.getPoints(shapeNo);
		int verts = rawShape.length;
		movingPoints = verts;
		
		// convert to a matrix and find the largest distance from a centre of the shape
		Matrix[] scaled = new Matrix[verts];
		double maxDist = 0.0;
		for (int v = 0; v < verts; v++)
		{
			scaled[v] = Util.makeMatrix(rawShape[v]);
			for (int d = 0; d < 3; d++)
			{
				double dist = Math.abs(rawShape[v][d]);
				if (dist > maxDist) maxDist = dist;
			}
		}
		
		// scale so that all vertices (positive and negative) fit in 0.5*shapeSize
		double scaler = 0.5*shapeSize/maxDist;
		for (int v = 0; v < verts; v++)
			scaled[v] = scaled[v].times(scaler);
		
		// make a random displacement of the shape centre from the centre of the volume
		boolean inSpace = false;
		while (!inSpace)
		{
			double edge = 0.1 + 0.5*shapeSize;
			double x = 2.0*Math.random();
			double y = 2.0*Math.random() - 1.0;
			double z = 2.0*Math.random() - 1.0;
			
			// x,y, and z should not be too near the edge
			inSpace =  (((y*y + z*z) < (1.0 - edge)*(1.0 - edge)) && (Math.abs(x - 1.0) < (1.0 - edge)));
			if (inSpace)
			{
				double[] pos = new double[3];
				pos[0] = x;
				pos[1] = y;
				pos[2] = z;
				
				//place the centre
				Matrix centre = Util.makeMatrix(pos);
				
				// place the vertices, unrotated
				Matrix[] vertices = new Matrix[verts];
				for (int v = 0; v < verts;v++)
					vertices[v] = centre.plus(scaled[v]);
				
				// random rotation about the centre of gravity
				vertices = randomRotate(vertices);
								
				// initially set the same vertex positions for all steps
				for (int step = 0; step < model.maxSteps; step++)
					for (int v = 0; v < verts;v++)
					{
						int obj = randomPoints + v;
						objectLocation[step][obj] = vertices[v];
					}
			}
		}
		
		// to ensure all objects are estimated, drawn, etc
		allPoints = randomPoints + verts;
		
		// make all points in the shape move

		// movement parameters set by sliders
		int firstStep = (int)(startStep);
		int nSteps = (int)(inSteps);
		
		// use the first object in the shape to define its direction of movement
		int firstObject = randomPoints;

		// move the first object in the shape towards the animal position at the first step - the hardest direction to detect
		Matrix startAnimal = Util.makeMatrix(animalPosition[firstStep]);
		Matrix diff = objectLocation[firstStep][firstObject].minus(startAnimal);
		double norm = Math.sqrt(diff.transpose().times(diff).get(0,0)); // to normalise the difference
		// total movement is a distance motion (set by slider)  in nSteps steps
		Matrix mStep = diff.times(-motion/(norm*nSteps)); // movement each step
		
		// movement for nSteps steps
		for (int ms = 0; ms < nSteps +1; ms++)
		{
			int step = firstStep + ms;
			for (int obj = randomPoints; obj < allPoints; obj++)
			    objectLocation[step][obj] = objectLocation[step-1][obj].plus(mStep);			
		}
		
		// stationary after that
		for (int step = firstStep + nSteps; step < model.maxSteps; step++)
			for (int obj = randomPoints; obj < allPoints; obj++)
				objectLocation[step][obj] = objectLocation[step-1][obj];			
				

		
		model.restart(true);
	}
	
	private Matrix[] randomRotate(Matrix[] oldVertices)
	{
		// make a copy to change
		Matrix[] vertices = new Matrix[oldVertices.length];
		for (int v = 0; v < vertices.length; v++) vertices[v]  = oldVertices[v];

		// scale to compute mean positions
		double scale = 1.0/vertices.length;

		// find the centre of gravity
		Matrix centre = Matrix.identity(3, 1).times(0.0);
		for (Matrix vertex: vertices) centre.plusEquals(vertex.times(scale));
		
		// move vertices to centre of gravity frame
		for (Matrix vertex: vertices) vertex.minusEquals(centre);
		
		// make a random rotation matrix
		Matrix rotator = rotation();
		
		// rotate vertices in the c of g frame
		for (int v = 0; v < vertices.length; v++) vertices[v] = rotator.times(vertices[v]);
		
		// add back the c of g
		for (Matrix vertex: vertices) vertex.plusEquals(centre);
		
		return vertices;
	}
	
	/**
	 * 
	 * @return random rotation matrix about two axes
	 */
	private Matrix rotation()
	{
		// rotation angle about y axis
		double yAng = Math.PI*Math.random();
		double cy = Math.cos(yAng);
		double sy = Math.sin(yAng);
		
		// rotation angle about z axis
		double zAng = Math.PI*Math.random();
		double cz = Math.cos(zAng);
		double sz = Math.sin(zAng);
		
		Matrix yRot = Matrix.identity(3, 3);
		yRot.set(0, 0, cy);
		yRot.set(2, 2, cy);
		yRot.set(2, 0, sy);
		yRot.set(0, 2, -sy);
		
		Matrix my = yRot.times(yRot.transpose());
		
		Matrix zRot = Matrix.identity(3, 3);
		zRot.set(0, 0, cz);
		zRot.set(1, 1, cz);
		zRot.set(1, 0, sz);
		zRot.set(0, 1, -sz);
		
		Matrix mz = zRot.times(zRot.transpose());
				
		Matrix rotator = yRot.times(zRot);
		
		Matrix check = rotator.times(rotator.transpose());
		
		return yRot.times(zRot);
	}
	
	
	public void makeAllAnimalSteps()
	{
		startAnimalPosition();
		
		for (int step = 1; step < model.maxSteps; step++)
			makeAnimalStep(step);
	}
	




}
