package utilPack;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Attr;
import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

import Jama.EigenvalueDecomposition;
import Jama.Matrix;

public class Util {
	

	//------------------------------------------------------------------------------------------
	//                                  RGB colours
	//------------------------------------------------------------------------------------------
	
	
	/**
	 * unpack a colour integer
	 * @param rgbInt
	 * @return
	 */
	public static int[] unpack(int rgb)
	{
		int[] pix = new int[3];
		// int a = rgb >> 32 & 0xff;
		pix[0] = rgb >> 16 & 0xff;
		pix[1] = rgb >> 8 & 0xff;
		pix[2] = rgb & 0xff;
		
		return pix;
	}
	
	/**
	 * pack a colour integer
	 * @param pix
	 * @return
	 */
	public static int pack(int[] pix)
	{
		return (pix[2] + 256*(pix[1] + 256*pix[0]));
	}

	//------------------------------------------------------------------------------------------
	//                           subsets of integers 0... n-1
	//------------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param m
	 * @param n
	 * @return all subsets of length m of the integers 0.. n - 1
	 * (each subset is in ascending order)
	 */
	public static Vector<int[]> subsets(int m, int n) throws Exception
	{
		if (m < 1) throw new Exception("For subsets of integers 1..n-1, m must be 1 or more"); 
		if (m > n) throw new Exception("For subsets of integers 1..n-1, m must not be more than n"); 
		Vector<int[]> subs = new Vector<int[]>();
		

		// for m = 1, the result is [0],[1],[2]...[n-1].
		if (m == 1)
		{
			for (int i = 0; i < n; i++)
			{
				int[] p = new int[1];
				p[0] = i;
				subs.add(p);
			}
		}
		
		// for  m > 1, extend the results for (m-1), where possible
		else if (m > 1)
		{
			// go over int[] in the result  for (m-1)
			Vector<int[]> ss = subsets(m-1,n);
			for (int[] spart: ss)
			{
				// last element of each int[];
				int last = spart[spart.length -1];
				// j takes all possible values for the last element of the new extension
				for (int j = last + 1; j < n; j++) // (there may be  no possible j)
				{
					int[] part = new int[m];
					// copy the previous int[]
					for (int k = 0; k < spart.length; k++) part[k] = spart[k];
					// add j on the end
					part[m-1] = j;
					subs.add(part);
				}
			}
		}
		
		
		return subs;
	}
	

	//------------------------------------------------------------------------------------------
	//                           Permutations 
	//------------------------------------------------------------------------------------------
	
	/**
	 * a slightly inefficient method, but so what?
	 * I expect to use it only up to about n = 20
	 * @param n
	 * @return a random permutation of [0...(n-1)]
	 */
	public static int[] randomPermutation(int n)
	{
		int[] perm = new int[n];
		for (int i = 0; i < n; i++) perm[i] = -1;
		int pos = 0;
		
		while (pos < n)
		{
			// a random element 0... n-1 in the array
			int c = (int)(Math.random()*n);
			
			// if this element has not yet been filled (increasingly unlikely)
			if (perm[c] == -1)
			{
				// fill the element with pos, and move pos up
				perm[c] = pos;
				pos++;
			}
		}
		
		return perm;
	}
	
	public static double factorial(int n) 
	{
		if (n < 2) return(1.0);
		else return n*factorial(n-1);
	}
	
	/**
	 * 
	 * @param <T>
	 * @param start
	 * @return
	 */
	public static <T> Vector<Vector<T>> permutations(Vector<T> start)
	{
		Vector<Vector<T>> result = new Vector<Vector<T>>();
		
		// lowest case - only one element to permute
		if (start.size() == 1)
		{
			result.add(start);
			return result;
		}
		
		// all longer cases
		for (int i = 0; i < start.size(); i++)
		{
			// extract an element, to put at the head of all permutations of the others
			T extracted = start.get(i);
			
			// make a Vector one element shorter, of all except the extracted element
			Vector<T> shorter = new Vector<T>();
			for (int j = 0; j < start.size(); j++)
				if (i!= j) shorter.add(start.get(j));
			
			// make all permutations of the shorter Vector
			Vector<Vector<T>> shorterPerms = permutations(shorter);
			
			// loop over all the shorter permutations
			for (Vector<T> shortPerm: shorterPerms)
			{
				// put the extracted element at the head of each shorter permutation
				Vector<T> full = new Vector<T>();
				full.add(extracted);
				for (T el:shortPerm) full.add(el);
				
				// add the full Vector to the result
				result.add(full);
			}
			
		}
		
		return result;
	}
	
	/**
	 * each pairing is a Vector of int[2], in the ranges [0..s1-1, 0..s2-1], with no repeats in either of the two positions.
	 * The length of each Vector is the smaller of s1 and s2.
	 * The ints are indexes of elements of two lists of FS, of length s1 and s2, to be generalised together.
	 * The shorter of the two lists must have all its elements generalised.
	 * So each Vector<int[]> has length min(s1,s2)
	 * if ordered is true, every int[] must have integers ascending
	 * 
	 * @param s1
	 * @param s2
	 * @return
	 */
	public static Vector<Vector<int[]>> pairings(int s1, int s2, boolean ordered)
	{
		Vector<Vector<int[]>> res = new Vector<Vector<int[]>>();
		
		// s2 is larger; each set of pairings has length s2
		if (s2 > s1)
		{
			Vector<int[]> perms = Util.subPerms(s2,s1,ordered);
			// each sub-permutation of s2 has length s1
			for (int[] perm: perms)
			{
				Vector<int[]> partial = new Vector<int[]>();
				for (int pos = 0; pos < perm.length; pos++)
				{
					int[] pair = new int[2];
					pair[0] = pos;
					pair[1] = perm[pos] - 1;
					partial.add(pair);
				}
				res.add(partial);
			}
		}
		
		// s1 = s2, or s1 is larger; each set of pairings has length s1
		else
		{
			Vector<int[]> perms = Util.subPerms(s1,s2,ordered);
			// each sub-permutation of s1 has length s2
			for (int[] perm: perms)
			{
				Vector<int[]> partial = new Vector<int[]>();
				for (int pos = 0; pos < perm.length; pos++)
				{
					int[] pair = new int[2];
					pair[0] = perm[pos] - 1;
					pair[1] = pos;
					partial.add(pair);
				}
				res.add(partial);
			}
		}
		
		
		return res;
	}
	
	/**
	 * max is greater than or equal to min.
	 * each int[] in the Vector result has length min, and is the first min numbers in a permutation of 1..max.
	 * No int[] is repeated.
	 * if ordered is true, each int[] must have integers in ascending order
	 * 
	 * @param max
	 * @param min
	 * @return
	 */
	public static Vector<int[]> subPerms(int max, int min, boolean ordered)
	{
		// get max! permutations of 1...max
		Vector<int[]> allPerms = intPerms(max);
		
		// store all sub-permutations of length min, removing duplicates
		Hashtable<int[],String> perms = new Hashtable<int[],String>();
		for (int[] fullPerm: allPerms)
		{
			int[] subPerm = new int[min];
			for (int i = 0; i < min; i++) subPerm[i] = fullPerm[i];
			if (!ordered) perms.put(subPerm, "1");
			if (isOrdered(subPerm)) perms.put(subPerm, "1");
		}
		
		// convert to a Vector
		Vector<int[]> res = new Vector<int[]>();
		for (Enumeration<int[]> en = perms.keys(); en.hasMoreElements();) 
			res.add(en.nextElement());
		return res;
	}
	
	/**
	 * true if a subset of a permutation is ordered
	 * @param perm
	 * @return
	 */
	private static boolean isOrdered(int[] perm)
	{
		boolean ordered = true;
		for (int i = 1; i < perm.length; i++)
			if (perm[i] < perm[i-1]) ordered = false;
		return ordered;
	}

	
	/**
	 * each int[] in the Vector result has length max, and is a permutation of the integers 1..max.
	 * So there are max! different int[].
	 * @param max
	 * @return
	 */
	public static Vector<int[]> intPerms(int max)
	{
		Vector<int[]> res = new Vector<int[]>();
		
		// lowest case; return a Vector with one element [1]
		if (max == 1)
		{
			int[] r1 = new int[1];
			r1[0] = 1;
			res.add(r1);
		}
		
		// recursion for any other case
		else if (max > 1)
		{
			// each int[] in last has (max -1) elements
			Vector<int[]> last = intPerms(max - 1);
			for (int[] p1: last)
			{
				// places where the number max may be interposed in the previous list
				for (int place = 0;place < max; place++)
				{
					int[] next = new int[max];
					for (int i = 0; i < max; i++)
					{
						if (i < place) next[i] = p1[i];
						else if (i == place) next[i] = max;
						else if (i > place) next[i] = p1[i-1];
					}
					res.add(next);
				}
			}
		}
		return res;
	}
	

	/**
	 * simple test of permutations
	 */
	public static void permuteTest()
	{
		message("\nPermutation test");
		
		Vector<String> tv = new Vector<String>();
		tv.add("a");
		tv.add("b");
		tv.add("c");
		tv.add("d");
		
		// all permutations of tv
		Vector<Vector<String>> perms = permutations(tv);
		
		// write out each permutation of tv on a line
		for (Vector<String> perm: perms)
		{
			String ord ="";
			for (String el: perm) ord = ord + el + " ";
			message(ord);
		}
		
		message("\n");
	}

	//------------------------------------------------------------------------------------------
	//                               Cartesian products
	//------------------------------------------------------------------------------------------
	
	
	/**
	 * 
	 * @param <T>
	 * @param input is a Vector of length N, whose Ith element (sublist) has MI elements.
	 * @return a Vector of length M1*M2*...MN.
	 * Each of its elements has length N, and each of those elements is a choice of one element from each sublist.
	 * All combinations of choices appear.
	 */
	public static <T> Vector<Vector<T>> cartesianProduct(Vector<Vector<T>> input)
	{
		return cartesian(0,input);		
	}
	
	/**
	 * 
	 * @param <T>
	 * @param i
	 * @param input
	 * @return
	 */
	private static <T> Vector<Vector<T>> cartesian(int i,Vector<Vector<T>> input)
	{
		Vector<T> thisLot = input.get(i);
		Vector<Vector<T>>  res = new Vector<Vector<T>>();
		
		if (i == input.size() - 1)
		{
			res.add(thisLot);
		}
		else if (i < input.size() - 1)
		{
			Vector<Vector<T>> next = cartesian(i+1,input);
			for (T thisOne: thisLot)
			{
				for (Vector<T> tail: next)
				{
					Vector<T> head = new Vector<T>();
					head.add(thisOne);
					for (T tailEl: tail) head.add(tailEl);
					res.add(head);					
				}
			}					
		}
		return res;
	}



	//------------------------------------------------------------------------------------------
	//               Using free orders to compute allowed permutations of child nodes
	//------------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param <T>
	 * @param start
	 * @param sequences groups of indexes of elements which must be ordered and contiguous in any permutation
	 * @return
	 */
	public static <T> Vector<Vector<T>> constrainedPermutations(Vector<T> start, Vector<int[]> sequences)  throws Exception
	{
		// make a Vector of Vectors, to permute.
		Vector<Vector<T>> toPermute = new Vector<Vector<T>>();
		
		// unless there are at least two sequences to permute, no permutations are possible
		if (sequences.size() < 2)
		{
			toPermute.add(start);
			return toPermute;
		}
		
		// record which indexes in start have been used
		Hashtable<Integer,String> used = new Hashtable<Integer,String>();

		// sequences of elements that must remain contiguous
		for (int[] seq:sequences)
		{
			Vector<T> group = new Vector<T>();
			for (int iv: seq)
			{
				used.put(Integer.valueOf(iv),"");
				// order values are 1..N, not 0..(N-1)
				int pos = iv - 1;
				if ((pos < 0) ||(pos > start.size() -1)) throw new Exception("Invalid position " + (pos + 1) + " in order list");
				group.add(start.get(pos));
			}
			toPermute.add(group);
		}
		
		// single elements not in any sequence 
		// (there should usually not  be any of these - but if there are, they can be put anywhere not in any group)
		for (int i = 0; i < start.size(); i++)
		{
			// elements of order are in the convention 1..N, not (0..(N-1))
			Integer iv = Integer.valueOf(i + 1);
			if (used.get(iv) == null)
			{
				Vector<T> soloGroup = new Vector<T>();
				soloGroup.add(start.get(i));
				toPermute.add(soloGroup);
			}
		}
		
		// permute the Vector of Vectors
		Vector<Vector<Vector<T>>> permuted = permutations(toPermute);
		
		// flatten each permutation
		Vector<Vector<T>> result = new Vector<Vector<T>>();
		for (Vector<Vector<T>> onePerm: permuted)
		{
			Vector<T> flatPerm = new Vector<T>();
			for (Vector<T> permGroup: onePerm)
				for (T el: permGroup) flatPerm.add(el);

			result.add(flatPerm);
		}
		
		return result;
		
	}


	//------------------------------------------------------------------------------------------
	//               Computing order constraints in unification and generalisation
	//------------------------------------------------------------------------------------------
	
	/**
	 * chenge the order constraints (to be put on a parent node) to reflect moving or removing one child node
	 * 
	 * @param orders Vector of all order constraints
	 * @param oldPos old position of this child in the list of children
	 * @param newPos new position of this child, if it has been added to the list
	 * @param added true if it has been added
	 * 
	 * @return new Vector of order constraints
	 */
	public static Vector<int[]> updateOrders(Vector<int[]> orders, int oldPos, int newPos, boolean added)
	{
		Vector<int[]> newOrders = new Vector<int[]>();
		
		for (int[] order:orders)
		{
			// does the child appear in this order array?
			boolean matched = false;
			for (int pos = 0; pos< order.length; pos++) if (oldPos + 1 == order[pos]) matched = true;
			
			// no match; add this order constraint unchanged
			if (!matched) newOrders.add(order);
			
			// matched and given a new position; substitute the new position for the old
			else if (added)
			{
				int[] newOrder = new int[order.length];
				for (int i = 0; i < order.length; i++)
				{
					// change from the old position to the new, in 1..N convention
					if (order[i] == oldPos + 1) newOrder[i] = newPos + 1;
					// no change
					else newOrder[i] = order[i];
				}
				newOrders.add(newOrder);
			}
			
			// matched and removed; remove from the order
			else if (!added)
			{
				// if the new order will have only one entry, it says nothing; do not add it
				if (order.length > 2)
				{
					// the order is going to be one shorter
					int[] newOrder = new int[order.length - 1]; // 2 or more
					int j = 0; // to track position in the new order
					for (int i = 0; i < order.length; i++)
					{
						// if this child has been removed, do not add it to the order
						if (order[i] == oldPos + 1) {}
						// keep other children in the order
						else if (order[i] != oldPos + 1) 
						{
							newOrder[j] = order[i];
							j++;
						}
					}
					newOrders.add(newOrder);
				}
			}
		}
				
		return newOrders;
	}
	
	
	
	
	
	
	

	//------------------------------------------------------------------------------------------
	//                           String utilities 
	//------------------------------------------------------------------------------------------
	
	/**
	 * round a double so it prints nicely
	 * @param d
	 * @param places
	 * @return
	 */
	public static double round(double d, int places)
	{
		double times = 1.0;
		for (int p = 0; p < places; p++) times = times*10.0;
		int rounded = (int)(d*times);
		
		return (double)(rounded/times);
	}
	

	
	/**
	 * 
	 * @param sv a Vector 'a', 'b', etc
	 * @return a string a-b- ....
	 */
	public static String serial(Vector<String> sv)
	{
		String r ="";		
		for (String s: sv) r = r + s + "-";		
		return r;
	}
	
	
	
	/**
	 * 
	 * @param s1
	 * @param s2
	 * @return true if s1 includes s2 as a substring
	 */
	public static boolean includes(Vector<String> s1, Vector<String> s2)
	{
		boolean includes = false;
		
		if (s1.size() > s2.size() - 1)
		{
			for (int i = 0; i < s1.size() - s2.size() + 1; i++) if (!includes)
			{
				includes = true;
				for (int j = 0; j < s2.size(); j++)
					if (!(s1.get(i + j).equals(s2.get(j)))) includes = false;
			}
		}
		
		
		return includes;
	}
	
	/**
	 * 
	 * @param exSounds sounds of a learning example
	 * @param allWords a set of words
	 * @param keepWords; if true keep the words in the partition
	 * 
	 * find the words which might be in the sounds of the example;
	 * then try out different subsets of the words to find the best partition 
	 * (which has the smallest number of sounds unmatched by words)
	 * @return
	 */
	public static Vector<Vector<String>>  bestPartition(Vector<String> exSounds, Vector<Vector<String>> allWords, boolean keepWords)
	{
		Vector<Vector<String>> split = new Vector<Vector<String>>();
		split.add(exSounds);
		
		// make a list of the words that might be in this example
		Vector<Vector<String>> pWords = new Vector<Vector<String>>();
		for (Vector<String> word: allWords)
			if (Util.includes(exSounds, word)) pWords.add(word);
		
		// the subset of these words that do not overlap with any other word in the list
		Vector<Vector<String>> qWords = new Vector<Vector<String>>();
		// the subset of words that overlap with some other word
		Vector<Vector<String>> rWords = new Vector<Vector<String>>();
		for (Vector<String> w: pWords)
		{
			boolean clean = true;
			// test overlap for all other words
			for (Vector<String> x: pWords) if (!Util.equalString(w, x))
			{
				if (endOverlap(w,x)) clean = false;
				if (endOverlap(x,w)) clean = false;
			}
			
			if (clean) qWords.add(w);
			else rWords.add(w);
		}
		
		// partition using the words with no overlap, that can always be used with no penalty
		for (Vector<String> w:qWords) split = partition(split, w, keepWords);
		
		// find the best split using some subset of the remaining words
		if (split.size() > 0) split = bestPartSplit(split, rWords,keepWords);
		
		return split;
	}
	
	/**
	 * 
	 * @param split a list of sequences of sounds
	 * @param rWords a set of words,  which overlap with one another
	 * 
	 * @return the best partition of split (which has the fewest unmatched sounds), using any subset of the words in rWords
	 */
	public static Vector<Vector<String>> bestPartSplit(Vector<Vector<String>> split, Vector<Vector<String>> rWords, boolean keepWord)
	{
		Vector<Vector<String>> best = new Vector<Vector<String>>();
		int bestScore = 1000;
		int s = rWords.size();
		int[] use = new int[s];
		int[] bestUse = new int[s];
		
		// the number of trials to do (= number of subsets of the words)
		int trials = 1;
		for (int i = 0; i < s; i++) trials = 2*trials;
		
		// do the trials (one for each subset of the words in rWords)
		for (int t = 0; t < trials; t++)
		{
			// clone the input split
			Vector<Vector<String>> sp = Util.vClone(split);
			
			// choose a subset of words to use in this trial
			int tp = t;
			for (int j = 0; j < s; j++)
			{
				if (tp > 2*(tp/2)) use[j] = 1; // if tp is odd
				else use[j] = 0;
				tp = tp/2;
			}
			
			// do the splitting with the chosen subset of words;
			// do not retain the words in this splitting result (if you did, all scores would be the same).
			for (int j = 0; j < s; j++) if (use[j] == 1)
				sp = partition(sp, rWords.get(j), false);
			
			// measure the result; score = number of unmatched sounds
			int score = 0;
			for (Vector<String> part:sp) score = score + part.size();
			
			// choose the splitting with the best score
			if (score < bestScore)
			{
				bestScore = score;
				best = sp;
				for (int i = 0; i < s; i++) bestUse[i] = use[i];
			}
		}
		
		// if you want the best partition with all the words retained, make it from the best split
		if (keepWord)
		{
			for (int j = 0; j < s; j++) if (bestUse[j] == 1)
				best = partition(split, rWords.get(j), true);
		}
		
		return best;
	}

	
	/**
	 * 
	 * @param wordSounds
	 * @ return wordSounds, removing any word which is a concatenation of two other words
	 */
	public static Vector<Vector<String>> removeConcatenations(Vector<Vector<String>> wordSounds)
	{
		Vector<Vector<String>> newSounds = new Vector<Vector<String>>();
		
		for (Vector<String> word: wordSounds)
		{
			boolean removeW = false;
			for (Vector<String> u: wordSounds) if (includes(word, u))
			{
				boolean startU = true;
				for (int i = 0; i < u.size(); i++) if(!(word.get(i).equals(u.get(i)))) startU = false;
				if (startU) for (Vector<String> v: wordSounds) 
					if ((Util.includes(word, v)) && (u.size() + v.size() == word.size()))
					{
						boolean endV = true;
						for (int i = 0; i < v.size(); i++) if(!(word.get(u.size() + i).equals(v.get(i)))) endV = false;
						if (endV) removeW = true;
					}				
			}
			
			if (!removeW) newSounds.add(word);
		}
		return newSounds;
	}

	
	
	
	/**
	 * 
	 * @param s1
	 * @param s2
	 * @return true of there is any overlap of the end of s1 with the start of s2
	 */
	public static boolean endOverlap(Vector<String> s1, Vector<String> s2)
	{
		boolean olap = false;
		// maximum number of characters that can overlap
		int max = Math.min(s1.size(), s2.size());
		
		// i = number of overlapping characters 
		for (int i = 1; i < max + 1; i++) if (!olap)
		{
			olap = true;
			// check for equality of all overlapping characters
			// j = position in s2; j - i < 0
			for (int j = 0; j < i; j++)
				if (!(s2.get(j).equals(s1.get(s1.size() + j - i)))) olap = false;
		}
		
		return olap;
	}
	
	/**
	 * Use a word w to further partition a list of sound sequences.
	 * For any sequence in the list, use the word to split the sequence, wherever the word occurs inside it
	 * (it may occur more than once in a sequence)
	 * 
	 * @param starter the list of sequences
	 * @param w the word
	 * @param keepWord if true, retain occurrences of the word as sequences
	 * @return
	 */
	public static Vector<Vector<String>> partition(Vector<Vector<String>> starter, Vector<String> w, boolean keepWord)
	{
		Vector<Vector<String>> res = new Vector<Vector<String>>();
		
		for (Vector<String> seq: starter)
		{
			if (!includes(seq, w)) res.add(seq);
			else
			{
				Vector<Vector<String>> r = splitPart(seq,w,keepWord);
				for (Vector<String> s:r) res.add(s);
			}
		}
		
		return res;
	}

	/**
	 * 
	 * @param seq a sequence of sounds
	 * @param w a word (sequence of sounds)
	 * @param keepword if true, retain the word in the result sequence
	 * @return the sequence split by the word, wherever it occurs
	 * (or not split, if the word is not contained in the sequence)
	 */
	public static Vector<Vector<String>> splitPart(Vector<String> seq, Vector<String> w, boolean keepWord)
	{
		Vector<Vector<String>> res = new Vector<Vector<String>>();
		
		// maximum possible start position for a match and split
		int max = seq.size() - w.size() + 1;
		
		int last = 0; // position of the last sound added to the result
		for (int s = 0; s < max; s++)
		{
			// check for a match starting at  position s
			boolean matches = true;
			for (int p = 0; p < w.size(); p++)
				if (!(w.get(p).equals(seq.get(s+p)))) matches = false;
			
			// if there is a match, add the section before the match, and maybe the word (if keepWord = true)
			if (matches)
			{
				if (s > last) // if there is any sequence to add before the word, add it
				{
					Vector<String> part = new Vector<String>();
					for (int i = last; i < s; i++) part.add(seq.get(i));
					res.add(part);
				}
				if (keepWord) res.add(w);
				
				// skip ahead to after the word
				s = s + w.size();
				// note what has been copied across
				last = s;
			}
			
			// if there is no match, just loop to try the next s
		}
		
		// if you have not consumed all the sounds, add a last sequence
		if (last < seq.size() -1)
		{
			Vector<String> part = new Vector<String>();
			for (int i = last; i < seq.size(); i++) part.add(seq.get(i));
			res.add(part);			
		}
		
		return res;
		
	}


	
	/**
	 * 
	 * @param svv list of lists or sounds
	 * @return clone of svv
	 */
	public static Vector<Vector<String>> vClone(Vector<Vector<String>> svv)
	{
		Vector<Vector<String>> vc = new Vector<Vector<String>>();
		for (Vector<String> sv: svv)
		{
			Vector<String> c = new  Vector<String>();
			for (String s:sv) c.add(s);
			vc.add(c);
		}
		return vc;
	}

	
	/**
	 * test for equality of two vectors of strings
	 * @param s1
	 * @param s2
	 * @return
	 */
	public static boolean equalString(Vector<String> s1, Vector<String> s2)
	{
		boolean eq = false;
		
		if (s1.size() == s2.size())
		{
			eq = true;
			for (int i = 0; i < s1.size(); i++)
				if(!(s1.get(i).equals(s2.get(i)))) eq = false;
		}
		
		return eq;		
	}
	
	/**
	 * 
	 * @param whole
	 * @param part
	 * @return true if whole starts with part
	 */
	public static boolean startsWith(Vector<String> whole, Vector<String> part)
	{
		boolean starts = false;
		if (whole.size() >= part.size())
		{
			starts = true;
			for (int p = 0; p < part.size(); p++)
				if (!whole.get(p).equals(part.get(p))) starts = false;
		}
		return starts;
	}
	
	/**
	 * 
	 * @param whole
	 * @param part
	 * @return true if whole ends with part
	 */
	public static boolean endsWith(Vector<String> whole, Vector<String> part)
	{
		boolean ends = false;
		if (whole.size() >= part.size())
		{
			ends = true;
			int d = whole.size() - part.size();
			for (int p = 0; p < part.size(); p++)
				if (!whole.get(p+d).equals(part.get(p))) ends = false;
		}
		return ends;
	}



	/**
	 * 
	 * @param s1 a list of sounds
	 * @param s2 a sub-list of sounds, contained in s1
	 * @return the index of s1 where s2 starts (for the first time)
	 */
	public static int startIndex(Vector<String> s1, Vector<String> s2)
	{
		boolean includes = false;
		int start = -1;
				
		for (int i = 0; i < s1.size() - s2.size() + 1; i++) if (!includes)
		{
			includes = true;
			for (int j = 0; j < s2.size(); j++)
				if (!(s1.get(i + j).equals(s2.get(j)))) includes = false;
			
			if (includes) start = i;
		}
				
		return start;
	}
	
	/**
	 * 
	 * @param s
	 * @param start
	 * @param end
	 * @return a sub-vector of s, from elements start to end inclusive
	 */
	public static Vector<String> subVector(Vector<String> s, int start, int end)
	{
		Vector<String> res = new Vector<String>();		
		for (int i = start; i < end + 1; i++) res.add(s.get(i));
		
		return res;
	}
	
	/**
	 * 
	 * @param vals
	 * @param val
	 * @return index of a string value in an array of  strings
	 * or -1 if it is not in the array
	 * @throws Exception
	 */
	public static int indexOf(String[] vals, String val)
	{
		int pos = -1;
		for (int i = 0; i < vals.length; i++)
			if (val.equals(vals[i])) pos = i;
		
		return pos;
	}
	
	/**
	 * 
	 * @param name
	 * @param names
	 * @return
	 */
	public static boolean inVector(String name, Vector<String> names)
	{
		boolean res = false;
		for (String cand:names) if (cand.equals(name)) res = true;
		return res;
	}
	
	/**
	 * 
	 * @param val
	 * @param vals
	 * @return
	 */
	public static boolean inArray(int val, int[] vals)
	{
		boolean res = false;
		if (vals != null) for (int v:vals) if (v == val) res = true;
		return res;
	}


	/**
	 * merge or add an index in an array, which may be empty (null)
	 * if the index is already in the array, make no change
	 * @param indices
	 * @param index
	 * @return
	 */
	public static int[] mergeInArray(int[] indices, int index)
	{
		int[] newInd = null;
		boolean found = false;

		//first instance of this sound sequence (eg trigram)
		if (indices == null)
		{
			newInd = new int[1];
			newInd[0] = index;
			found = true;
		}
		
		// instance of this sound sequence may have been found already
		else if (indices != null)
		{
			for (int i = 0; i < indices.length; i++)
				if (index == indices[i]) found = true;
			if (found) newInd = indices;
		}
		
		// new instance of this sound sequence (eg trigram)
		if ((indices != null) && (!found))
		{
			newInd = new int[indices.length + 1];
			for (int j = 0; j < indices.length; j++) newInd[j] = indices[j];
			newInd[indices.length] = index;
		}
		
		return newInd;
		
	}

	/**
	 * 
	 * @param text
	 * @return
	 */
	public static String removeSpaces(String text)
	{
		String noSpace = "";
		char[] ca = new char[1];
		
		for (int c = 0; c < text.length(); c++)
		{
			char ch = text.charAt(c);
			ca[0] = ch;
			if (ch != ' ') noSpace = noSpace + new String(ca);
		}
		
		return noSpace;
	}
	
	
	/**
	 * 
	 * @param ss
	 * @return
	 */
	public static String oneString(Vector<String> ss)
	{
		String one = "";
		for (String s:ss) one = one + "." + s;
		return one;
	}


	//------------------------------------------------------------------------------------------
	//                           XML - Getting elements 
	//-------------------------------------------------------------------------------------------
	

    /**
     * Vector of child elements of an element
     */
    public static Vector<Element> childElements(Element el) {
        Vector<Element> res = new Vector<Element>();
        NodeList nodes = el.getChildNodes();
        for (int i = 0; i < nodes.getLength(); i++) {
            Node nd = nodes.item(i);
            if (nd instanceof Element) {
                Element eg = (Element) nd;
                res.addElement(eg);
            }
        }
        return res;
    }
    
    /**
     * return the Vector of child Elements with given local name
     * @param el
     * @param name
     * @return
     */
    public static Vector<Element> namedChildElements(Element el, String lName)
    {
    	Vector<Element> nc = new Vector<Element>();
    	for (Iterator<Element> it = childElements(el).iterator(); it.hasNext();)
    	{
    		Element en = it.next();
    		if (getLocalName(en).equals(lName)) nc.addElement(en);
    	}
    	return nc;
    }
    
    /**
     * 
     * @param el
     * @param lName
     * @return the first child element with given name
     * @throws Exception if there are none
     */
    public static Element firstNamedChild(Element el, String lName) throws Exception
    {
    	Vector<Element> els = namedChildElements( el, lName);
    	if (els.size() == 0) throw new Exception("Element " + getLocalName(el) + " has no child element " + lName);
    	return els.get(0);
    }
    
    /**
 	 * get the name of an XML element, with the namespace prefix stripped off
 	 * @param el
 	 * @return
 	 */
 	public static String getLocalName(Node el)
 	{
 		String locName = "";
 		StringTokenizer st = new StringTokenizer(el.getNodeName(),":");
 		while (st.hasMoreTokens()) locName = st.nextToken();
 		return locName;
 	}
 	
	/**
	 * 
	 * @param fromEl
	 * @return a table <attribute name, attribute value>
	 */
  	public static Hashtable<String,String> getAllAttributes(Element fromEl)
	{
  		Hashtable<String,String> allAtts = new Hashtable<String,String>();
  		
		for (int a = 0; a < fromEl.getAttributes().getLength();a++)
		{
			Attr at = (Attr)fromEl.getAttributes().item(a);
			allAtts.put(at.getName(), at.getValue());
		}
		
		return allAtts;
	}
  	
    /**
     * get the  text string in an element (eg interspersed between child elements), 
     * or "" if there is none or if the Element is null.
     * Tries to ignore white space text; but does not succeed.
     */
    public static String getText(Element el) {
        String res = "";
        if (el != null) try 
        {
        	el.normalize(); // does not help recognise white space
        	NodeList nodes = el.getChildNodes();
            for (int i = 0; i < nodes.getLength(); i++) 
            	if (nodes.item(i) instanceof Text)
            	{
            		Text text = (Text)nodes.item(i);
            		// this filter seems to make no difference
            		if (!text.isElementContentWhitespace())  
            		{
            			String tData = text.getData();
            			// this seems to be an effective way to catch pure white space
            			StringTokenizer nonWhiteSpace = new StringTokenizer(tData,"\n \t");
            			if (nonWhiteSpace.countTokens() > 0 )res = res + tData;
            		}
            	}
        }
        catch (Exception e) {System.out.println("Text failure: " + e.getMessage());}
        return res;
    }


	//------------------------------------------------------------------------------------------
	//                                  Reading XML files
	//-------------------------------------------------------------------------------------------
	
	/**
	 * return the root Element of an XML file with given location
	 * @param location
	 * @return Element the root element, or null if there is any problem
	 */
	public static Element getRootElement(String location) throws Exception
	{
		Element root = null;
        try {
            FileInputStream fi = new FileInputStream(location);
            DocumentBuilderFactory builderFac = DocumentBuilderFactory.newInstance();
            builderFac.setNamespaceAware(true);
            root = builderFac.newDocumentBuilder().parse(fi).getDocumentElement();
            fi.close();
        }
        catch (SAXException ex) {notify(location,ex);}
        catch (FileNotFoundException ex) {notify(location,ex);}
        catch (IOException ex) {notify(location,ex);}
        catch (ParserConfigurationException ex) {notify(location,ex);}
		return root;
	}
	
	/**
	 * return the root Element of an XML file in a given inputStream
	 * @param location
	 * @return Element the root element, or null if there is any problem
	 */
	public static Element getRootElement(InputStream fi) throws Exception
	{
		return getDocument(fi).getDocumentElement();
	}
	
	/**
	 * return the Document of an XML file in a given inputStream
	 * @param location
	 * @return Element the root element, or null if there is any problem
	 */
	public static Document getDocument(InputStream fi) throws Exception
	{
		Document doc = null;
        try {
            DocumentBuilderFactory builderFac = DocumentBuilderFactory.newInstance();
            builderFac.setNamespaceAware(true);
            doc = builderFac.newDocumentBuilder().parse(fi);
        }
        catch (SAXException ex) {notify("Input Stream",ex);}
        catch (FileNotFoundException ex) {notify("Input Stream",ex);}
        catch (IOException ex) {notify("Input Stream",ex);}
        catch (ParserConfigurationException ex) {notify("Input Stream",ex);}
		return doc;
	}
	
	/**
	 * return the Document of an XML file in a given inputStream
	 * @param location
	 * @return Element the root element, or null if there is any problem
	 */
	public static Document getDocument(String location) throws Exception
	{
		try{
            FileInputStream fi = new FileInputStream(location);
            return getDocument(fi);
		}
        catch (IOException ex) {notify("Input Stream",ex);}
        return null;
	}


	private static void notify(String location, Exception ex) throws Exception
	{
		throw new Exception ("Exception getting XML root element from "
				+ location + "; " + ex.getMessage());
	}


	//------------------------------------------------------------------------------------------
	//                           XML - making output documents
	//-------------------------------------------------------------------------------------------
	

    /**
     * create a new element in no namespace with no text content.
     * write an error message and return null if any exception
     *
     * @param name String: name of the element
     * @return Element: the element
     */
    public static Element newElement(Document  outDoc, String name) 
    throws Exception {
        Element ei = null;
        try {
            ei = (Element) outDoc.createElement(name);
        }
        catch (Exception e) {
            throw new Exception ("Exception creating element '" + name
                    + "'. " + e.getMessage());
        }
        return ei;
    }

    /**
     * create a new element in no namespace with no text content.
     * write an error message and return null if any exception
     *
     * @param name String: name of the element
     * @param text String: the text content
     * @return Element: the element
     */
    public static Element textElement(Document outDoc, String name, String text) 
    throws Exception {
        Text t;
        Element en = null;
        try {
            en = newElement(outDoc,name);
            t = outDoc.createTextNode(text);
            en.appendChild(t);
        }
        catch (Exception e) {
      	  throw new Exception ("Exception creating text-filled element '" + name
                    + "'. " + e.getMessage());
        }
        return en;
    }

    /**
     * add a comment below element el
     */
    public static void addComment(Document outDoc, Element el, String cText) {
        Comment com = outDoc.createComment(cText);
        el.appendChild(com);
    }


	//--------------------------------------------------------------------------------------
    //	                              Output File handling
	//--------------------------------------------------------------------------------------

	    // make an output XML document

	    public static Document makeOutDoc() throws Exception
	    {
	    	Document outDoc = null;
	        try {
	            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	            dbf.setNamespaceAware(true);
	            DocumentBuilder db = dbf.newDocumentBuilder();
	            outDoc = db.newDocument();
	        }
	        catch (ParserConfigurationException pce) {
	            throw new Exception("Parser Config exception: " + pce.getMessage());
	        }
	        return outDoc;
	    }



	    /**
	     * write the XML output to the named file
	     * @param fileName the filename to write to
	     * @param outDoc the XML document to be written
	     * @param isFormatted if true, put in line breaks and indenting
	     * @throws XMLException if anything goes wrong
	     */
	    public static void writeOutput(Document outDoc, String fileName, boolean isFormatted)
	    throws Exception
	    {
	        if (isFormatted) {
	            addFormatting(outDoc);
	        }
	        try {
	            TransformerFactory tf = TransformerFactory.newInstance();
	            Transformer t = tf.newTransformer();
	            Source src = new DOMSource(outDoc);
	            Result dest = new StreamResult(new File(fileName));
	            t.transform(src, dest);
	        }
	        catch (TransformerConfigurationException tce) {
	        	throw new Exception("Transform Config exception: " + tce.getMessage());
	        }
	        catch (TransformerException te) {
	            throw new Exception("Transform exception: " + te.getMessage());
	        }
	    }


	    /**
	     * write the XML output to the output stream
	     * @param Document Doc the XML document to be written out
	     * @param OutputStream stream the output stream to write it to
	     * @param boolean formatted : true if you want sensible line breaks and indents
	     */
	    public static void writeToStream(Document Doc, OutputStream stream, boolean formatted) 
	    throws Exception
	    {
	        if (formatted) {
	            addFormatting(Doc,Doc.getDocumentElement(),"");
	        }
	        try {
	            TransformerFactory tf = TransformerFactory.newInstance();
	            Transformer t = tf.newTransformer();
	            Source src = new DOMSource(Doc);
	            Result dest = new StreamResult(stream);
	            t.transform(src, dest);
	        }
	        catch (TransformerConfigurationException tce) {
	            throw new Exception("Transform Config exception: " + tce.getMessage());
	        }
	        catch (TransformerException te) {
	        	throw new Exception("Transform exception: " + te.getMessage());
	        }
	    }

	    /**
	    * write the XML output to the named file
	    * @param fileName the filename to write to
	    * @param xslFilename the file to use to transform
	    */
	   public static void writeFormattedOutput(Document outDoc, String fileName, String xslFilename )
	   throws Exception 
	   {
	       try {
	           StreamSource xsltSource = new StreamSource(new FileInputStream(xslFilename));
	           Result dest = new StreamResult(new File(fileName));
	           writeFormattedOutputFromXSLSource( outDoc, dest,  xsltSource);
	       } catch (FileNotFoundException e) {
	           throw new Exception("Transform file not found exception: " + e.getMessage());
	       }
	   }

	    /**
	    * write the XML output to the named file
	    * @param fileName the filename to write to
	    * @param xslStream input stream of the xsl  to use to transform
	    */
	   public static void writeTransformedOutput(Document outDoc, String fileName, InputStream xslStream )
	   throws Exception 
	   {
	           StreamSource xsltSource = new StreamSource(xslStream);
	           Result dest = new StreamResult(new File(fileName));
	           writeFormattedOutputFromXSLSource( outDoc, dest,  xsltSource);
	   }


	   /**
	   * create the XSLT transformed output
	   * @param xslNode the top node of the XSL DOM file used to transform
	   * @return the top node of the transformed DOM
	   */
	  public static Node makeTransformedOutput(Document outDoc, Node xslNode ) throws Exception
	  {
	          DOMSource xsltSource = new DOMSource(xslNode);
	          DOMResult dest = new DOMResult();
	          writeFormattedOutputFromXSLSource(outDoc,dest, xsltSource);
	          return dest.getNode();
	  }

	    /**
	     * write the XML output to the named file
	     * @param dest the Result to write to
	     * @param xsltSource the xslt to use to transform
	     */
	    public static void writeFormattedOutputFromXSLSource(Document outDoc, Result dest, Source xsltSource ) 
	    throws Exception
	    {
	        try {
	            TransformerFactory tf = TransformerFactory.newInstance();
	            Source src = new DOMSource(outDoc);
	            Templates xslt = tf.newTemplates(xsltSource);
	            // Apply the xsl file to the source file and write the result to the output file
	            Transformer trans = xslt.newTransformer();
	            // Transform the XML
	            trans.transform(src, dest);
	        }
	        catch (TransformerConfigurationException tce) {
	            throw new Exception("Transform Config exception: " + tce.getMessage());
	        }
	        catch (TransformerException te) {
	            throw new Exception("Transform exception: " + te.getMessage());
	        }
	    }

	    //---------------------------------------------------------------------------------------------------
	    //                                        formatting code
	    //---------------------------------------------------------------------------------------------------


	    /** This method goes through and adds formatting as necessary so that the XML is human readable
	     * TODO work out why we can't put space in front of the first element */
	    protected static void addFormatting(Document outDoc) {
	        // First we want a space before the first element
	        addFormatting(outDoc,outDoc.getDocumentElement(), "");

	    }

	    /** Goes through and adds newlines and indent to the current node and all its children
	     * @param current the current node
	     * @param indent the current level of indent this is increased recursively*/
	    private static void addFormatting(Document doc, Node current, String indent) {

	        // go through each of the children adding space as required
	        Node child = current.getFirstChild();
	        // any text node is taken to be a new line
	        boolean newLineBefore = (child.getNodeType() == Node.TEXT_NODE);
	        String childIndent = indent+"\t";
	        while (child != null) {
	            Node nextChild = child.getNextSibling();
	            if (child.getNodeType() != Node.TEXT_NODE) {
	                // Only if we aren't a text node, and there is no preceding text node (new line)  already, do we add the new line
	                if (!newLineBefore) current.insertBefore(doc.createTextNode("\n"+childIndent), child);
	                if (child.hasChildNodes()) {
	                    addFormatting(doc, child, childIndent);
	                }
	                if (nextChild == null) {
	                    // Because this is the last child, we need to add some space after it
	                    current.appendChild(doc.createTextNode("\n"+indent));
	                }
	            }
		        newLineBefore = (child.getNodeType() == Node.TEXT_NODE);
	            child = nextChild;
	        }
	    }
		
		//--------------------------------------------------------------------------------------------------
		//                                         reading text files
		//--------------------------------------------------------------------------------------------------
		
		/**
		 * read a text file at an absolute file location
		 */
		public static FileInputStream getTextFile(String location) throws Exception
		{
			FileInputStream fi = null;
			try {
				fi = new FileInputStream(location);
			}
			catch (Exception ex) {throw new Exception("Cannot read text file at '" + location + "': " + ex.getMessage());}
			return fi;
		}
		
		/**
		 * 
		 * @param location an absolute file location
		 * @return the lines of a text file at the location
		 * @throws MapperException
		 */
		public static Vector<String> textLines(String location) throws Exception
		{
			Vector<String> lines = new Vector<String>();
			FileInputStream fiz = getTextFile(location);
	        InputStreamReader isr = new InputStreamReader(fiz);

	        LineNumberReader lnr = new LineNumberReader(isr);
	        try {
	            String line = lnr.readLine();
	            while (line != null)
	            {
	                lines.add(line);
	                line = lnr.readLine();
	            }
	            lnr.close();        	
	        }
			catch (Exception ex) {throw new Exception("Failure reading read text file at '" + location + "': " + ex.getMessage());}

	        return lines;
		}

	    //--------------------------------------------------------------------------------------
	    //                              dialogs
	    //--------------------------------------------------------------------------------------
		
		/**
		 * 
		 * @param title
		 * @return what the user entered, or null if they cancelled
		 */
		public static String getShortUserInput(String title,JFrame frame, String previous)
		{
			String s = (String)JOptionPane.showInputDialog(
                    frame,
                    null,
                    title,
                    JOptionPane.PLAIN_MESSAGE,
                    null,
                    null,
                    previous);
			return s;
		}
		
		/**
		 * 
		 * @param title
		 * @return true if the user chooses yes
		 */
		public static boolean getYesNo(String title, JFrame frame)
		{
			Object[] options = {"Yes", "No" };
			int choice  = JOptionPane.showOptionDialog(
					frame, 
					null, 
					title,
			        JOptionPane.YES_NO_OPTION, 
			        JOptionPane.PLAIN_MESSAGE,
			        null, 
			        options, 
			        null);
			
			return (choice == 0);
		}
	    
	
	//--------------------------------------------------------------------------------------------------
	//                                         writing text files
	//--------------------------------------------------------------------------------------------------
	
	public static void writeTextFile(String location, Vector<String> textRows) throws Exception
	{
		try
		{
	    	FileOutputStream fo = new FileOutputStream(location);
	    	for (int i = 0; i < textRows.size(); i++)
	    	{
	    		String row = textRows.get(i);
	    		nl(row, fo);
	    	}
	    	fo.close();	
		}
		catch (Exception ex) {ex.printStackTrace();throw new Exception(ex.getMessage());}		
	}
	
	
	private static byte[] newLine = {13,10};

    // write out a String followed by a new line
	public static  void nl(String s, OutputStream fo)
    {
        try{
        fo.write(s.getBytes());
        fo.write(newLine);
        }
        catch (IOException e)
            {message("Exception writing line '" + s + "': " + e.getMessage());}
    }

    // write out a String followed by no new line
    public static  void nnl(String s, OutputStream fo)
    {
        try{
        fo.write(s.getBytes());
        }
        catch (IOException e)
            {message("Exception writing line '" + s + "': " + e.getMessage());}
    }

    public static  void quoted(String s, FileOutputStream fo)
    {
        byte[] quote = {'"'};
        try{
            fo.write(quote);
            fo.write(s.getBytes());
            fo.write(quote);
        }
        catch (IOException e)
            {message("Exception writing quoted string '" + s + "': " + e.getMessage());}
    } 

    //--------------------------------------------------------------------------------------
    //                                memory management
    //--------------------------------------------------------------------------------------
	
    
    /**
     * 
     * @param place
     * @param index
     */
    public static void macroMemoryMessage(String place, int index)
    {
        Runtime rt = Runtime.getRuntime();

        int m1  = (int)(rt.maxMemory()/1000000.0);
        int t1  = (int)(rt.totalMemory()/1000000.0);
        int f1  = (int)(rt.freeMemory()/1000000.0);
        int u1  = t1 - f1;

        rt.gc();
 
        int m2  = (int)(rt.maxMemory()/1000000.0);
        int t2  = (int)(rt.totalMemory()/1000000.0);
        int f2  = (int)(rt.freeMemory()/1000000.0);
        int u2  = t1 - f1;
        
        message("\nMemory MB at " + place + "("  + index + ") before gc - max:" + m1 + "; total: " + t1 + "; used: "   + u1 + "; free: " + f1);
        message("After gc - max:" + m2 + "; total: " + t2 + "; used: "   + u2 + "; free: " + f2);
            
    }
    
    /**
     * 
     * @param index
     */
    public static void showMem(int index)
    {
        Runtime rt = Runtime.getRuntime();

        int m1  = (int)rt.maxMemory();
        int t1  = (int)rt.totalMemory();
        int f1  = (int)rt.freeMemory();
        int u1  = t1 - f1;
        message("Memory at ("  + index + ") - max:" + m1 + "; total: " + t1 + "; used: "   + u1 + "; free: " + f1);    	
    }
    
    /**
     * find free memory before some operation, to see what memory it uses
     * @return
     */
    public static int freeBefore()
    {
        Runtime rt = Runtime.getRuntime();
        int f1  = (int)rt.freeMemory();
        return f1;    	
    }
    
    /**
     * write out the free memory used by some operation
     * @param before
     * @param place
     */
    public static void freeUsed(int before, String place)
    {
        Runtime rt = Runtime.getRuntime();
        String garbage = "";
        int used  = before - (int)rt.freeMemory();
        if (used > 1000000)
        {
        	rt.gc();
            used  = before - (int)rt.freeMemory();
        	garbage = "(gc)";
        }
        if (used != 0) message("Free memory used in " + place + garbage + " : " + used + " bytes");
    }

    /**
     * 
     * @return
     */
    public static long memory()
    {
        Runtime rt = Runtime.getRuntime();
        rt.gc();
        long used = rt.totalMemory() - rt.freeMemory();
        return used;
    }

    //--------------------------------------------------------------------------------------
    //                      Poisson-distributed random integers
    //--------------------------------------------------------------------------------------
    
    /**
     * If an integer has a Poisson distribution with mean m,
     * the probability of it having value i is  m**i * exp(-m)/m!, 
     * 
     * so the sum of probabilities for all i 
     * is exp(-m) * (1 + m + m**2/2 + m**3/6....) = 1
     * @param mean
     * @return
     */
    public static int poisson(double m)
    {
    	double prob = Math.random();
    	double pSum = 0.0;
    	double ex = Math.exp(-m);
    	double pow = 1.0;   // goes 1, m, m**2, .... as i = 0,1,2,...
    	double fact = 1.0;  // goes 1,1,2,6,24... as i = 0,1,2...
    	
    	for (int i = 0; i < 30; i++)
    	{
    		pSum = pSum + pow*ex/fact;
    		pow = pow*m;
    		fact = fact*(i+1);
    		if (pSum > prob) return i;
    	}
    	
    	// as m is typically much smaller than 30, should never happen
    	return 30;
    }
	

    //--------------------------------------------------------------------------------------
    //                                Geometry
    //--------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param va homogeneous 2-Vector
	 * @param vb homogeneous 2-Vector
	 * @return cross product of two homogeneous 2-vectors, not normalised
	 */
	public static double[] cross(double[] va, double[] vb)
	{
		double[] w = new double[3];
		
		for (int i = 0; i < 3; i++)
		{
			w[i] = 0.0;
			for (int j = 0; j < 3; j++)
				for (int k = 0; k < 3; k++)
					w[i] = w[i] + epsilon(i,j,k)*va[j]*vb[k];
		}
				
		// w should be orthogonal to both v1 and v2, in 3 dimensions
		if (dot3(w,w) > 0.000001)
		{
			double dotCheck = dot3(w,va)/dot3(w,w);
			if (Math.abs(dotCheck) > 0.01) 
			{
				Util.message("Cross product failure " + dotCheck);
			}
		}
				
		return w;
	}
	
	/**
	 * 
	 * @param i
	 * @param j
	 * @param k
	 * @return
	 */
	public static double epsilon(int i, int j, int k)
	{
		double eps = 0.0;
		
		for (int ind = 0; ind < nonZero.length; ind++)
		{
			int[] term = nonZero[ind];
			if ((term[0] == i) && (term[1] == j) && (term[2] == k)) eps = (double)term[3];
		}
		
		return eps;
	}
	
	
	// 6 non-zero values of epsilon(i,j,k) as {i,j,k,value}
	private static int[][] nonZero = {
			{0,1,2,1},
			{0,2,1,-1},
			{1,2,0,1},
			{1,0,2,-1},
			{2,0,1,1},
			{2,1,0,-1}};
	
	
	
	/**
	 * 
	 * @param va
	 * @param vb
	 * @return dot product of two homogeneous 3-vectors, not normalised
	 */
	public static double dot3(double[] va, double[] vb)
	{
		double raw = (va[0]*vb[0] + va[1]*vb[1] + va[2]*vb[2]);
		
		// do not normalise the result
		return raw;
	}
	
	/**
	 * 
	 * @param v1
	 * @param v2
	 * @return dot product of two homogeneous 2-vectors, normalised
	 */
	public static double dot2(double[] v1, double[] v2)
	{
		double raw = (v1[0]*v2[0] + v1[1]*v2[1]);
		
		// normalise the result
		return raw/(v1[2]*v2[2]);
	}
	
	/**
	 * distance between two homogeneous points
	 * @param v1 homogeneous 2-Vector
	 * @param v2 homogeneous 2-Vector
	 * @return
	 */
	public static double dist(double[] v1, double[] v2)
	{
		double d2 = 0.0;
		
		// loop over (x,y)
		for (int dim = 0; dim < 2; dim++)
		{
			// scale the points to scale 1.0
			double r1 = v1[dim]/v1[2];
			double r2 = v2[dim]/v2[2];
			
			d2 = d2 + (r1-r2)*(r1 - r2);
		}
		
		return (Math.sqrt(d2));
	}

	//------------------------------------------------------------------------------
	//                             3-D matrices and vectors
	//------------------------------------------------------------------------------
	/**
	 * generate a random Gaussian between -4.0 and 4.0
	 * with standard deviation 1.
	 * @return
	 */
	public static double randomGaussian()
	{
		double val = 0.0;
		boolean found = false;
		
		while (!found)
		{
			
			//wide trial range -4 to 4
			double x = -4.0 + 8.0*Math.random();
			// standard deviation 1.0
			double g = Math.exp(-x*x/2.0);			
			double y = Math.random();

			if (g > y)
			{
				val = x;
				found = true;
			}			
		}
		return val;
	}
	
	/**
	 * 
	 * @param v
	 * @return
	 */
	public static double[] makeVec(Matrix v)
	{
		double[]vec = new double[3];
		if (v!= null) for (int dim = 0; dim <3; dim++) vec[dim] = v.get(dim,0);
		else
		{
			Util.message("Null vector matrix");
		}
		return vec;
	}
	
	
	/**
	 * 
	 * @param v
	 * @return
	 */
	public static Matrix makeMatrix(double[]v)
	{
		Matrix res = new Matrix(3,1);
		for (int dim = 0; dim <3; dim++) res.set(dim, 0, v[dim]);
		return res;
	}
	

	
	/**
	 * cross product of two 3-vectors, represented as [3][1] matrices
	 * @param m1
	 * @param m2
	 * @return
	 */
	public static Matrix crossProduct(Matrix m1, Matrix m2)
	{
		double[][] array = new double[3][1];
		
		array[0][0] = m1.get(1, 0)*m2.get(2,0) - m1.get(2, 0)*m2.get(1,0);
		array[1][0] = m1.get(2, 0)*m2.get(0,0) - m1.get(0, 0)*m2.get(2,0);
		array[2][0] = m1.get(0, 0)*m2.get(1,0) - m1.get(1, 0)*m2.get(0,0);
		
		return new Matrix(array);
	}
	
	/**
	 * 
	 * @param mat
	 * @param name
	 */
	public static void writeMatrix(double[][] mat, String name)
	{
		Util.message("\n" + name + " : \n\tx\ty\tz");
		for (int d = 0; d < mat.length; d++)
		{
			double [] row = mat[d];
			String mLine = "";
			for (int e = 0; e < row.length; e++) 
				mLine = mLine + "\t" + round(row[e], 4);
			message(mLine);
		}
	}
	
	/**
	 * string to write out all elements of a matrix
	 * @param mat
	 * @return
	 */
	public static String mString(Matrix mat)
	{
		String mLine = "";
		
		for (int r = 0; r < mat.getRowDimension();r++)
			for (int c = 0; c < mat.getColumnDimension();c++)
				mLine = mLine + Double.valueOf(round(mat.get(r, c),4)) + "\t";
		
		
		return mLine;
		
	}
	
	

    //--------------------------------------------------------------------------------------
    //                             Eigenvectors and eigenvalues, with a check
    //--------------------------------------------------------------------------------------
	
	/**
	 * 
	 * @param moments a 3*3 symmetric matrix
	 * @param check
	 * @returns 3*3  [eigenvalues, eigenvectors]
	 * eigenvalues are diagonal elements.
	 * eigenvectors are columns (second index selects an eigenvector)
	 */
	public static Matrix[] eigenCheck(Matrix moments, boolean check)
	{
		boolean testFreeze = false;
		
		if (testFreeze)
		{
			// this trace has proved that it sometimes freezes making the EigenvalueDecomposition
			message("eigenCheck");

			// freezing is not caused by a null matrix
			if (moments == null) message("null matrix");

			// freezing is not caused by a small determinant
			double det = moments.det();
			if (Math.abs(det) < 0.00001) message("determinant " + det);
		}

		EigenvalueDecomposition eComp = moments.eig();
		Matrix eVals = eComp.getD();	
		Matrix eVecs = eComp.getV();
		
		if (check)
		{
			message("\nCheck of eigenvectors");
			for (int ev = 0; ev < 3; ev++)
			{
				double eValue = eVals.get(ev,ev);
				// this matrix (M - lambda*I) should give zero when multiplying the correct eigenvector
				Matrix sub = moments.minus(Matrix.identity(3, 3).times(eValue));
				
				// extract the correct eigenvector,from column ev
				Matrix eVector = new Matrix(3,1);
				for (int dim = 0; dim < 3; dim++) eVector.set(dim, 0, eVecs.get(dim, ev));
				
				// this should be a zero vector
				Matrix mult = sub.times(eVector);

				String checkLine = "eigenvalue: " + eValue + "\t";
				for (int d = 0; d <3; d++)
					checkLine = checkLine + round(mult.get(d, 0),6) + "\t";
				message(checkLine);
			}
		}
		

		Matrix[] result = new Matrix[2];
		result[0] = eVals;
		result[1] = eVecs;		
		return result;
	}
	
	/**
	 * 
	 * @param mat
	 * @param name
	 */
	public static void checkOrtho(Matrix mat, String name)
	{
		Util.message("\nChecking orthogonality of matrix " +  name);
		for (int bar = 0; bar < 3; bar++)
		{
			String row = "";
			for (int car = 0; car < 3; car++)
			{
				// scalar product of two vectors, in columns bar and car
				double dotp = 0.0;
				for (int d = 0; d< 3; d++)
					dotp = dotp + mat.get(d,bar)*  mat.get(d,car);

				row = row + Util.round(dotp, 6) + "\t";
			}
			Util.message(row);			
		}
	}


	    //--------------------------------------------------------------------------------------
	    //                                  trivia
	    //--------------------------------------------------------------------------------------
		
		/**
		 * 
		 * @param vec
		 * @return the Vector of Strings, reversed
		 */
		public static Vector<String> reverse(Vector<String> vec)
		{
			Vector<String> result = new Vector<String>();
			
			for (int i = 0; i < vec.size(); i++) 
				result.add(vec.get(vec.size() - i - 1));
			
			return result;
		}
	    
	    /**
	     * message to console
	     * @param s
	     */
		public static void message(String s) {System.out.println(s);}
	    


}
